# -*- coding: utf-8 -*-
#
# This file is part of Linux Show Player
#
# Copyright 2012-2016 Francesco Ceruti <ceppofrancy@gmail.com>
#
# Linux Show Player is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Linux Show Player is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Linux Show Player.  If not, see <http://www.gnu.org/licenses/>.

from PyQt5.QtCore import Qt, QT_TRANSLATE_NOOP
from PyQt5.QtWidgets import QVBoxLayout, QSizePolicy, QDialogButtonBox,\
    QDialog, QAbstractItemView, QHeaderView, QTableView

from lisp.application import Application
from lisp.core.has_properties import Property
from lisp.cues.cue import Cue, CueAction
from lisp.ui.cuelistdialog import CueSelectDialog
from lisp.ui.qdelegates import CueActionDelegate, CueSelectionDelegate
from lisp.ui.qmodels import CueClassRole, SimpleCueListModel
from lisp.ui.settings.cue_settings import CueSettingsRegistry
from lisp.ui.settings.settings_page import SettingsPage
from lisp.ui.ui_utils import translate


class CollectionCue(Cue):
    Name = QT_TRANSLATE_NOOP('CueName', 'Collection Cue')

    targets = Property(default=[])

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.name = translate('CueName', self.Name)

    def __start__(self, fade=False):
        for target_id, action in self.targets:
            cue = Application().cue_model.get(target_id)
            if cue is not self:
                cue.execute(action=CueAction[action])

        return False


class CollectionCueSettings(SettingsPage):
    Name = QT_TRANSLATE_NOOP('SettingsPageName', 'Edit Collection')

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.setLayout(QVBoxLayout(self))

        self.cue_dialog = CueSelectDialog(
            cues=Application().cue_model,
            selection_mode=QAbstractItemView.ExtendedSelection)

        self.collectionModel = CollectionModel()
        self.collectionView = CollectionView(self.cue_dialog, parent=self)
        self.collectionView.setModel(self.collectionModel)
        self.collectionView.setAlternatingRowColors(True)
        self.layout().addWidget(self.collectionView)

        # Buttons
        self.dialogButtons = QDialogButtonBox(self)
        self.dialogButtons.setSizePolicy(QSizePolicy.Minimum,
                                         QSizePolicy.Minimum)
        self.layout().addWidget(self.dialogButtons)

        self.addButton = self.dialogButtons.addButton(
            translate('CollectionCue', 'Add'), QDialogButtonBox.ActionRole)
        self.addButton.clicked.connect(self._add_dialog)

        self.delButton = self.dialogButtons.addButton(
            translate('CollectionCue', 'Remove'), QDialogButtonBox.ActionRole)
        self.delButton.clicked.connect(self._remove_selected)

    def load_settings(self, settings):
        for target_id, action in settings.get('targets', []):
            target = Application().cue_model.get(target_id)
            if target is not None:
                self._add_cue(target, CueAction(action))

    def get_settings(self):
        targets = []
        for target_id, action in self.collectionModel.rows:
            targets.append((target_id, action.value))

        return {'targets': targets}

    def _add_cue(self, cue, action):
        self.collectionModel.appendRow(cue.__class__, cue.id, action)
        self.cue_dialog.remove_cue(cue)

    def _add_dialog(self):
        if self.cue_dialog.exec_() == QDialog.Accepted:
            for target in self.cue_dialog.selected_cues():
                self._add_cue(target, target.CueActions[0])

    def _remove_selected(self):
        row = self.collectionView.currentIndex().row()
        cue_id = self.collectionModel.rows[row][0]

        self.collectionModel.removeRow(row)
        self.cue_dialog.add_cue(Application().cue_model.get(cue_id))


class CollectionView(QTableView):
    def __init__(self, cue_select, **kwargs):
        super().__init__(**kwargs)

        self.setSelectionBehavior(QTableView.SelectRows)
        self.setSelectionMode(QTableView.SingleSelection)

        self.setShowGrid(False)
        self.setAlternatingRowColors(True)

        self.horizontalHeader().setSectionResizeMode(QHeaderView.Stretch)
        self.horizontalHeader().setHighlightSections(False)

        self.verticalHeader().sectionResizeMode(QHeaderView.Fixed)
        self.verticalHeader().setDefaultSectionSize(26)
        self.verticalHeader().setHighlightSections(False)

        self.delegates = [
            CueSelectionDelegate(cue_select),
            CueActionDelegate()
        ]

        for column, delegate in enumerate(self.delegates):
            self.setItemDelegateForColumn(column, delegate)


class CollectionModel(SimpleCueListModel):
    def __init__(self):
        # NOTE: The model does fixed-indices operations based on this list
        super().__init__([translate('CollectionCue', 'Cue'),
                          translate('CollectionCue', 'Action')])

    def setData(self, index, value, role=Qt.DisplayRole):
        result = super().setData(index, value, role)

        if result and role == CueClassRole:
            if self.rows[index.row()][1] not in value.CueActions:
                self.rows[index.row()][1] = value.CueActions[0]
                self.dataChanged.emit(self.index(index.row(), 1),
                                      self.index(index.row(), 1),
                                      [Qt.DisplayRole, Qt.EditRole])

        return result


CueSettingsRegistry().add_item(CollectionCueSettings, CollectionCue)



# -*- coding: utf-8 -*-
#
# This file is part of Linux Show Player
#
# Copyright 2012-2016 Francesco Ceruti <ceppofrancy@gmail.com>
#
# Linux Show Player is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Linux Show Player is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Linux Show Player.  If not, see <http://www.gnu.org/licenses/>.

from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QGroupBox, QVBoxLayout, QCheckBox, QGridLayout, \
    QSpinBox, QLabel

from lisp.ui.settings.settings_page import SettingsPage
from lisp.ui.ui_utils import translate


class CartLayoutSettings(SettingsPage):

    Name = 'Cart Layout'

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.setLayout(QVBoxLayout())
        self.layout().setAlignment(Qt.AlignTop)

        self.behaviorsGroup = QGroupBox(self)
        self.behaviorsGroup.setLayout(QVBoxLayout())
        self.layout().addWidget(self.behaviorsGroup)

        self.countdownMode = QCheckBox(self.behaviorsGroup)
        self.behaviorsGroup.layout().addWidget(self.countdownMode)

        self.showSeek = QCheckBox(self.behaviorsGroup)
        self.behaviorsGroup.layout().addWidget(self.showSeek)

        self.showDbMeters = QCheckBox(self.behaviorsGroup)
        self.behaviorsGroup.layout().addWidget(self.showDbMeters)

        self.showAccurate = QCheckBox(self.behaviorsGroup)
        self.behaviorsGroup.layout().addWidget(self.showAccurate)

        self.showVolume = QCheckBox(self.behaviorsGroup)
        self.behaviorsGroup.layout().addWidget(self.showVolume)

        self.autoAddPage = QCheckBox(self.behaviorsGroup)
        self.behaviorsGroup.layout().addWidget(self.autoAddPage)

        self.gridSizeGroup = QGroupBox(self)
        self.gridSizeGroup.setLayout(QGridLayout())
        self.layout().addWidget(self.gridSizeGroup)

        self.columnsSpin = QSpinBox(self.gridSizeGroup)
        self.columnsSpin.setRange(1, 16)
        self.gridSizeGroup.layout().addWidget(self.columnsSpin, 0, 0)

        self.columnsLabel = QLabel(self.gridSizeGroup)
        self.gridSizeGroup.layout().addWidget(self.columnsLabel, 0, 1)

        self.rowsSpin = QSpinBox(self.gridSizeGroup)
        self.rowsSpin.setRange(1, 16)
        self.gridSizeGroup.layout().addWidget(self.rowsSpin, 1, 0)

        self.rowsLabel = QLabel(self.gridSizeGroup)
        self.gridSizeGroup.layout().addWidget(self.rowsLabel, 1, 1)

        self.gridSizeGroup.layout().setColumnStretch(0, 5)
        self.gridSizeGroup.layout().setColumnStretch(1, 3)

        self.retranslateUi()

    def retranslateUi(self):
        self.behaviorsGroup.setTitle(
            translate('CartLayout', 'Default behaviors'))
        self.countdownMode.setText(translate('CartLayout', 'Countdown mode'))
        self.showSeek.setText(translate('CartLayout', 'Show seek-bars'))
        self.showDbMeters.setText(translate('CartLayout', 'Show dB-meters'))
        self.showAccurate.setText(translate('CartLayout', 'Show accurate time'))
        self.showVolume.setText(translate('CartLayout', 'Show volume'))
        self.autoAddPage.setText(
            translate('CartLayout', 'Automatically add new page'))
        self.gridSizeGroup.setTitle(translate('CartLayout', 'Grid size'))
        self.columnsLabel.setText(translate('CartLayout', 'Number of columns'))
        self.rowsLabel.setText(translate('CartLayout', 'Number of rows'))

    def get_settings(self):
        conf = {
            'gridcolumns': str(self.columnsSpin.value()),
            'gridrows': str(self.rowsSpin.value()),
            'showdbmeters': str(self.showDbMeters.isChecked()),
            'showseek': str(self.showSeek.isChecked()),
            'showaccurate': str(self.showAccurate.isChecked()),
            'showvolume': str(self.showVolume.isChecked()),
            'countdown': str(self.countdownMode.isChecked()),
            'autoaddpage': str(self.autoAddPage.isChecked())
        }

        return {'CartLayout': conf}

    def load_settings(self, settings):
        settings = settings.get('CartLayout', {})
        if 'gridcolumns' in settings:
            self.columnsSpin.setValue(int(settings['gridcolumns']))
        if 'gridrows' in settings:
            self.rowsSpin.setValue(int(settings['gridrows']))
        if 'showseek' in settings:
            self.showSeek.setChecked(settings['showseek'] == 'True')
        if 'showdbmeters' in settings:
            self.showDbMeters.setChecked(settings['showdbmeters'] == 'True')
        if 'showaccurate' in settings:
            self.showAccurate.setChecked(settings['showaccurate'] == 'True')
        if 'showvolume' in settings:
            self.showVolume.setChecked(settings['showvolume'] == 'True')
        if 'countdown' in settings:
            self.countdownMode.setChecked(settings['countdown'] == 'True')
        if 'autoaddpage' in settings:
            self.autoAddPage.setChecked(settings['autoaddpage'] == 'True')



# -*- coding: utf-8 -*-
#
# This file is part of Linux Show Player
#
# Copyright 2012-2016 Francesco Ceruti <ceppofrancy@gmail.com>
#
# Linux Show Player is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Linux Show Player is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Linux Show Player.  If not, see <http://www.gnu.org/licenses/>.

from PyQt5.QtCore import Qt, QT_TRANSLATE_NOOP
from PyQt5.QtWidgets import QGroupBox, QPushButton, QComboBox, QVBoxLayout, \
    QMessageBox, QTableView, QTableWidget, QHeaderView, QGridLayout

from lisp.modules import check_module
from lisp.modules.midi.midi_input import MIDIInput
from lisp.plugins.controller.protocols.protocol import Protocol
from lisp.ui.qdelegates import ComboBoxDelegate, SpinBoxDelegate, \
    CueActionDelegate
from lisp.ui.qmodels import SimpleTableModel
from lisp.ui.settings.settings_page import CueSettingsPage
from lisp.ui.ui_utils import translate


class Midi(Protocol):
    def __init__(self):
        super().__init__()

        if check_module('midi'):
            midi_input = MIDIInput()
            if not midi_input.is_open():
                midi_input.open()
            MIDIInput().new_message.connect(self.__new_message)

    def __new_message(self, message):
        if message.type == 'note_on' or message.type == 'note_off':
            self.protocol_event.emit(Midi.str_from_message(message))

    @staticmethod
    def str_from_message(message):
        return Midi.str_from_values(message.type, message.channel, message.note)

    @staticmethod
    def str_from_values(m_type, channel, note):
        return '{} {} {}'.format(m_type, channel, note)

    @staticmethod
    def from_string(message_str):
        m_type, channel, note = message_str.split()
        return m_type, int(channel), int(note)


class MidiSettings(CueSettingsPage):
    Name = QT_TRANSLATE_NOOP('SettingsPageName', 'MIDI Controls')

    def __init__(self, cue_class, **kwargs):
        super().__init__(cue_class, **kwargs)
        self.setLayout(QVBoxLayout())
        self.layout().setAlignment(Qt.AlignTop)

        self.midiGroup = QGroupBox(self)
        self.midiGroup.setTitle(translate('ControllerMidiSettings', 'MIDI'))
        # self.midiGroup.setEnabled(check_module('midi'))
        self.midiGroup.setLayout(QGridLayout())
        self.layout().addWidget(self.midiGroup)

        self.midiModel = SimpleTableModel([
            translate('ControllerMidiSettings', 'Type'),
            translate('ControllerMidiSettings', 'Channel'),
            translate('ControllerMidiSettings', 'Note'),
            translate('ControllerMidiSettings', 'Action')])

        self.midiView = MidiView(cue_class, parent=self.midiGroup)
        self.midiView.setModel(self.midiModel)
        self.midiGroup.layout().addWidget(self.midiView, 0, 0, 1, 2)

        self.addButton = QPushButton(self.midiGroup)
        self.addButton.clicked.connect(self.__new_message)
        self.midiGroup.layout().addWidget(self.addButton, 1, 0)

        self.removeButton = QPushButton(self.midiGroup)
        self.removeButton.clicked.connect(self.__remove_message)
        self.midiGroup.layout().addWidget(self.removeButton, 1, 1)

        self.midiCapture = QPushButton(self.midiGroup)
        self.midiCapture.clicked.connect(self.capture_message)
        self.midiGroup.layout().addWidget(self.midiCapture, 2, 0)

        self.msgTypeCombo = QComboBox(self.midiGroup)
        self.msgTypeCombo.addItem(
            translate('ControllerMidiSettings', 'Filter "note on"'))
        self.msgTypeCombo.setItemData(0, 'note_on', Qt.UserRole)
        self.msgTypeCombo.addItem(
            translate('ControllerMidiSettings', 'Filter "note off"'))
        self.msgTypeCombo.setItemData(1, 'note_off', Qt.UserRole)
        self.midiGroup.layout().addWidget(self.msgTypeCombo, 2, 1)

        self.retranslateUi()

        self._default_action = self._cue_class.CueActions[0].name

    def retranslateUi(self):
        self.addButton.setText(translate('ControllerSettings', 'Add'))
        self.removeButton.setText(translate('ControllerSettings', 'Remove'))
        self.midiCapture.setText(translate('ControllerMidiSettings', 'Capture'))

    def enable_check(self, enabled):
        self.midiGroup.setCheckable(enabled)
        self.midiGroup.setChecked(False)

    def get_settings(self):
        settings = {}
        checkable = self.midiGroup.isCheckable()

        if not (checkable and not self.midiGroup.isChecked()):
            messages = []

            for row in self.midiModel.rows:
                message = Midi.str_from_values(row[0], row[1]-1, row[2])
                messages.append((message, row[-1]))

            if messages:
                settings['midi'] = messages

        return settings

    def load_settings(self, settings):
        if 'midi' in settings:
            for options in settings['midi']:
                m_type, channel, note = Midi.from_string(options[0])
                self.midiModel.appendRow(m_type, channel+1, note, options[1])

    def capture_message(self):
        handler = MIDIInput()
        handler.alternate_mode = True
        handler.new_message_alt.connect(self.__add_message)

        QMessageBox.information(self, '',
                                translate('ControllerMidiSettings',
                                          'Listening MIDI messages ...'))

        handler.new_message_alt.disconnect(self.__add_message)
        handler.alternate_mode = False

    def __add_message(self, msg):
        if self.msgTypeCombo.currentData(Qt.UserRole) == msg.type:
            self.midiModel.appendRow(msg.type, msg.channel+1, msg.note,
                                     self._default_action)

    def __new_message(self):
        message_type = self.msgTypeCombo.currentData(Qt.UserRole)
        self.midiModel.appendRow(message_type, 1, 0, self._default_action)

    def __remove_message(self):
        self.midiModel.removeRow(self.midiView.currentIndex().row())


class MidiView(QTableView):
    def __init__(self, cue_class, **kwargs):
        super().__init__(**kwargs)

        self.delegates = [
            ComboBoxDelegate(options=['note_on', 'note_off']),
            SpinBoxDelegate(minimum=1, maximum=16),
            SpinBoxDelegate(minimum=0, maximum=127),
            CueActionDelegate(cue_class=cue_class,
                              mode=CueActionDelegate.Mode.Name)
        ]

        self.setSelectionBehavior(QTableWidget.SelectRows)
        self.setSelectionMode(QTableView.SingleSelection)

        self.setShowGrid(False)
        self.setAlternatingRowColors(True)

        self.horizontalHeader().setSectionResizeMode(QHeaderView.Stretch)
        self.horizontalHeader().setHighlightSections(False)

        self.verticalHeader().sectionResizeMode(QHeaderView.Fixed)
        self.verticalHeader().setDefaultSectionSize(24)
        self.verticalHeader().setHighlightSections(False)

        for column, delegate in enumerate(self.delegates):
            self.setItemDelegateForColumn(column, delegate)



# -*- coding: utf-8 -*-
#
# This file is part of Linux Show Player
#
# Copyright 2012-2016 Francesco Ceruti <ceppofrancy@gmail.com>
#
# Linux Show Player is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Linux Show Player is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Linux Show Player.  If not, see <http://www.gnu.org/licenses/>.

from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QDialog, QHBoxLayout, QListWidget, QVBoxLayout, \
    QPushButton, QDialogButtonBox, QInputDialog, QMessageBox

from lisp.core.configuration import config
from lisp.core.util import compose_http_url
from lisp.modules.remote.remote import RemoteController
from lisp.ui import elogging
from lisp.ui.ui_utils import translate
from .peers_discovery_dialog import PeersDiscoveryDialog


class PeersDialog(QDialog):
    def __init__(self, peers, **kwargs):
        super().__init__(**kwargs)
        self.peers = peers

        self.setWindowModality(Qt.ApplicationModal)
        self.setMaximumSize(500, 200)
        self.setMinimumSize(500, 200)
        self.resize(500, 200)

        self.setLayout(QHBoxLayout())

        self.listWidget = QListWidget(self)
        self.listWidget.setAlternatingRowColors(True)
        self.layout().addWidget(self.listWidget)

        for peer in self.peers:
            self.listWidget.addItem(peer['uri'])

        self.buttonsLayout = QVBoxLayout()
        self.layout().addLayout(self.buttonsLayout)

        self.discoverPeersButton = QPushButton(self)
        self.addPeerButton = QPushButton(self)
        self.removePeerButton = QPushButton(self)
        self.removeAllButton = QPushButton(self)

        self.discoverPeersButton.clicked.connect(self.discover_peers)
        self.addPeerButton.clicked.connect(self.add_peer)
        self.removePeerButton.clicked.connect(self.remove_peer)
        self.removeAllButton.clicked.connect(self.remove_all)

        self.buttonsLayout.addWidget(self.discoverPeersButton)
        self.buttonsLayout.addWidget(self.addPeerButton)
        self.buttonsLayout.addWidget(self.removePeerButton)
        self.buttonsLayout.addWidget(self.removeAllButton)
        self.buttonsLayout.addSpacing(70)

        self.dialogButton = QDialogButtonBox(self)
        self.dialogButton.setStandardButtons(self.dialogButton.Ok)
        self.dialogButton.accepted.connect(self.accept)
        self.buttonsLayout.addWidget(self.dialogButton)

        self.layout().setStretch(0, 2)
        self.layout().setStretch(1, 1)

        self.retranslateUi()

    def retranslateUi(self):
        self.setWindowTitle(
            translate('SyncPeerDialog', 'Manage connected peers'))
        self.discoverPeersButton.setText(
            translate('SyncPeerDialog', 'Discover peers'))
        self.addPeerButton.setText(
            translate('SyncPeerDialog', 'Manually add a peer'))
        self.removePeerButton.setText(
            translate('SyncPeerDialog', 'Remove selected peer'))
        self.removeAllButton.setText(
            translate('SyncPeerDialog', 'Remove all peers'))

    def add_peer(self):
        ip, ok = QInputDialog.getText(self,
                                      translate('SyncPeerDialog', 'Address'),
                                      translate('SyncPeerDialog', 'Peer IP'))
        if ok:
            self._add_peer(ip)

    def _add_peer(self, ip):
        port = config['Remote']['BindPort']
        uri = compose_http_url(ip, port)

        for peer in self.peers:
            if peer['uri'] == uri:
                QMessageBox.critical(self,
                                     translate('SyncPeerDialog', 'Error'),
                                     translate('SyncPeerDialog',
                                               'Already connected'))
                return

        try:
            peer = {'proxy': RemoteController.connect_to(uri), 'uri': uri}
            self.peers.append(peer)
            self.listWidget.addItem(peer['uri'])
        except Exception as e:
            elogging.exception(translate('SyncPeerDialog', 'Cannot add peer'),
                               str(e))

    def discover_peers(self):
        dialog = PeersDiscoveryDialog(parent=self)
        if dialog.exec_() == dialog.Accepted:
            for peer in dialog.get_peers():
                self._add_peer(peer)

    def remove_peer(self):
        if len(self.listWidget.selectedIndexes()) != 0:
            self.peers.pop(self.current_index())
            self.listWidget.takeItem(self.current_index())

    def remove_all(self):
        self.peers.clear()
        self.listWidget.clear()

    def current_index(self):
        return self.listWidget.selectedIndexes()[0].row()



# -*- coding: utf-8 -*-
#
# This file is part of Linux Show Player
#
# Copyright 2012-2016 Francesco Ceruti <ceppofrancy@gmail.com>
#
# Linux Show Player is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Linux Show Player is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Linux Show Player.  If not, see <http://www.gnu.org/licenses/>.

from PyQt5 import QtCore
from PyQt5.QtGui import QPainter
from PyQt5.QtWidgets import QLabel


class QVerticalLabel(QLabel):

    def paintEvent(self, event):
        painter = QPainter(self)
        painter.rotate(-90)

        region = QtCore.QRect(-self.height(), 0, self.height(), self.width())
        hint = painter.drawText(region, self.alignment(), self.text())

        painter.end()

        self.setMaximumWidth(hint.height())
        self.setMinimumWidth(0)
        self.setMaximumHeight(16777215)
        self.setMinimumHeight(hint.width())

    def sizeHint(self):
        return self.size()



# -*- coding: utf-8 -*-
#
# This file is part of Linux Show Player
#
# Copyright 2012-2016 Francesco Ceruti <ceppofrancy@gmail.com>
#
# Linux Show Player is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Linux Show Player is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Linux Show Player.  If not, see <http://www.gnu.org/licenses/>.

from PyQt5.QtCore import Qt, QT_TRANSLATE_NOOP
from PyQt5.QtWidgets import QGroupBox, QVBoxLayout, QComboBox, QGridLayout, QLabel

from lisp.modules import check_module
from lisp.modules.midi.midi_input import MIDIInput
from lisp.modules.midi.midi_output import MIDIOutput
from lisp.modules.midi.midi_utils import mido_backend
from lisp.ui.settings.settings_page import SettingsPage
from lisp.ui.ui_utils import translate


class MIDISettings(SettingsPage):
    Name = QT_TRANSLATE_NOOP('SettingsPageName', 'MIDI settings')

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.setLayout(QVBoxLayout())
        self.layout().setAlignment(Qt.AlignTop)

        self.midiGroup = QGroupBox(self)
        self.midiGroup.setTitle(
            translate('MIDISettings', 'MIDI default devices'))
        self.midiGroup.setLayout(QGridLayout())
        self.layout().addWidget(self.midiGroup)

        self.inputLabel = QLabel(translate('MIDISettings', 'Input'),
                                 self.midiGroup)
        self.midiGroup.layout().addWidget(self.inputLabel, 0, 0)
        self.inputCombo = QComboBox(self.midiGroup)
        self.midiGroup.layout().addWidget(self.inputCombo, 0, 1)

        self.outputLabel = QLabel(translate('MIDISettings', 'Output'),
                                  self.midiGroup)
        self.midiGroup.layout().addWidget(self.outputLabel, 1, 0)
        self.outputCombo = QComboBox(self.midiGroup)
        self.midiGroup.layout().addWidget(self.outputCombo, 1, 1)

        self.midiGroup.layout().setColumnStretch(0, 2)
        self.midiGroup.layout().setColumnStretch(1, 3)

        if check_module('Midi'):
            try:
                self._load_devices()
            except Exception:
                self.setEnabled(False)
        else:
            self.setEnabled(False)

    def get_settings(self):
        conf = {}

        if self.isEnabled():
            conf['inputdevice'] = self.inputCombo.currentText()
            MIDIInput().change_port(conf['inputdevice'])
        if self.isEnabled():
            conf['outputdevice'] = self.outputCombo.currentText()
            MIDIOutput().change_port(conf['outputdevice'])

        return {'MIDI': conf}

    def load_settings(self, settings):
        if 'inputdevice' in settings['MIDI']:
            self.inputCombo.setCurrentText('AppDefault')
            self.inputCombo.setCurrentText(settings['MIDI']['inputdevice'])

        if 'outputdevice' in settings['MIDI']:
            self.outputCombo.setCurrentText('AppDefaut')
            self.outputCombo.setCurrentText(settings['MIDI']['outputdevice'])

    def _load_devices(self):
        backend = mido_backend()

        self.inputCombo.clear()
        self.inputCombo.addItems(['AppDefault', 'SysDefault'])
        self.inputCombo.addItems(backend.get_input_names())

        self.outputCombo.clear()
        self.outputCombo.addItems(['AppDefault', 'SysDefault'])
        self.outputCombo.addItems(backend.get_output_names())



# -*- coding: utf-8 -*-
#
# This file is part of Linux Show Player
#
# Copyright 2012-2016 Francesco Ceruti <ceppofrancy@gmail.com>
#
# Linux Show Player is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Linux Show Player is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Linux Show Player.  If not, see <http://www.gnu.org/licenses/>.

from abc import abstractmethod
from enum import Enum

from lisp.core.has_properties import HasProperties, Property
from lisp.core.signal import Signal


class MediaState(Enum):
    """Identify the current media state"""
    Error = -1
    Null = 0
    Playing = 1
    Paused = 2
    Stopped = 3


class Media(HasProperties):
    """Interface for Media objects.

    Media(s) provides control over multimedia contents.
    To control various parameter of the media, MediaElement(s) should be used.

    .. note::
        The play/stop/pause functions must be non-blocking functions.
    """

    loop = Property(default=0)
    duration = Property(default=0)
    start_time = Property(default=0)
    stop_time = Property(default=0)

    def __init__(self):
        super().__init__()

        self.paused = Signal()
        """Emitted when paused (self)"""
        self.played = Signal()
        """Emitted when played (self)"""
        self.stopped = Signal()
        """Emitted when stopped (self)"""
        self.interrupted = Signal()
        """Emitted after interruption (self)"""
        self.eos = Signal()
        """End-of-Stream (self)"""

        self.on_play = Signal()
        """Emitted before play (self)"""
        self.on_stop = Signal()
        """Emitted before stop (self)"""
        self.on_pause = Signal()
        """Emitted before pause (self)"""

        self.sought = Signal()
        """Emitted after a seek (self, position)"""
        self.error = Signal()
        """Emitted when an error occurs (self, error, details)"""

        self.elements_changed = Signal()
        """Emitted when one or more elements are added/removed (self)"""

    @property
    @abstractmethod
    def state(self):
        """
        :return: the media current state
        :rtype: MediaState
        """

    @abstractmethod
    def current_time(self):
        """
        :return: the current playback time in milliseconds or 0
        :rtype: int
        """

    @abstractmethod
    def element(self, class_name):
        """
        :param name: The element class-name
        :type name: str

        :return: The element with the specified class-name or None
        :rtype: lisp.core.base.media_element.MediaElement
        """

    @abstractmethod
    def elements(self):
        """
        :return: All the MediaElement(s) of the media
        :rtype: list
        """

    @abstractmethod
    def elements_properties(self):
        """
        :return: Media elements configurations
        :rtype: dict
        """

    @abstractmethod
    def input_uri(self):
        """
        :return: The media input uri (e.g. "file:///home/..."), or None
        :rtype: str
        """

    @abstractmethod
    def interrupt(self):
        """Interrupt the playback (no fade) and go in STOPPED state."""

    @abstractmethod
    def pause(self):
        """The media go in PAUSED state and pause the playback."""

    @abstractmethod
    def play(self):
        """The media go in PLAYING state and starts the playback."""

    @abstractmethod
    def seek(self, position):
        """Seek to the specified point.

        :param position: The position to be reached in milliseconds
        :type position: int
        """

    @abstractmethod
    def stop(self):
        """The media go in STOPPED state and stop the playback."""

    @abstractmethod
    def update_elements(self, settings):
        """Update the elements configuration.

        :param settings: Media-elements settings
        :type settings: dict
        """


# -*- coding: utf-8 -*-
#
# This file is part of Linux Show Player
#
# Copyright 2012-2017 Francesco Ceruti <ceppofrancy@gmail.com>
#
# Linux Show Player is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Linux Show Player is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Linux Show Player.  If not, see <http://www.gnu.org/licenses/>.

from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QDoubleSpinBox, QGridLayout, QLabel, QWidget

from lisp.ui.ui_utils import translate
from lisp.ui.widgets import FadeComboBox


class FadeEdit(QWidget):

    def __init__(self, *args, mode=FadeComboBox.Mode.FadeOut, **kwargs):
        super().__init__(*args, **kwargs)
        self.setLayout(QGridLayout())

        self.fadeDurationSpin = QDoubleSpinBox(self)
        self.fadeDurationSpin.setRange(0, 3600)
        self.layout().addWidget(self.fadeDurationSpin, 0, 0)

        self.fadeDurationLabel = QLabel(self)
        self.fadeDurationLabel.setAlignment(Qt.AlignCenter)
        self.layout().addWidget(self.fadeDurationLabel, 0, 1)

        self.fadeTypeCombo = FadeComboBox(self, mode=mode)
        self.layout().addWidget(self.fadeTypeCombo, 1, 0)

        self.fadeTypeLabel = QLabel(self)
        self.fadeTypeLabel.setAlignment(Qt.AlignCenter)
        self.layout().addWidget(self.fadeTypeLabel, 1, 1)

        self.retranslateUi()

    def retranslateUi(self):
        self.fadeDurationLabel.setText(translate('FadeEdit', 'Duration (sec)'))
        self.fadeTypeLabel.setText(translate('FadeEdit', 'Curve'))

    def duration(self):
        return self.fadeDurationSpin.value()

    def setDuration(self, value):
        self.fadeDurationSpin.setValue(value)

    def fadeType(self):
        return self.fadeTypeCombo.currentType()

    def setFadeType(self, fade_type):
        self.fadeTypeCombo.setCurrentType(fade_type)



# -*- coding: utf-8 -*-
#
# This file is part of Linux Show Player
#
# Copyright 2012-2017 Francesco Ceruti <ceppofrancy@gmail.com>
#
# Linux Show Player is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Linux Show Player is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Linux Show Player.  If not, see <http://www.gnu.org/licenses/>.

from PyQt5.QtCore import QT_TRANSLATE_NOOP, Qt
from PyQt5.QtWidgets import QVBoxLayout, QGroupBox

from lisp.ui.settings.settings_page import SettingsPage
from lisp.ui.ui_utils import translate
from lisp.ui.widgets.fade_edit import FadeEdit


class CueAppSettings(SettingsPage):
    Name = QT_TRANSLATE_NOOP('SettingsPageName', 'Cue Settings')

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.setLayout(QVBoxLayout())
        self.layout().setAlignment(Qt.AlignTop)

        # Interrupt
        self.interruptGroup = QGroupBox(self)
        self.interruptGroup.setLayout(QVBoxLayout())
        self.layout().addWidget(self.interruptGroup)

        self.interruptFadeEdit = FadeEdit(self.interruptGroup)
        self.interruptGroup.layout().addWidget(self.interruptFadeEdit)

        # Action
        self.actionGroup = QGroupBox(self)
        self.actionGroup.setLayout(QVBoxLayout())
        self.layout().addWidget(self.actionGroup)

        self.fadeActionEdit = FadeEdit(self.actionGroup)
        self.actionGroup.layout().addWidget(self.fadeActionEdit)

        self.retranslateUi()

    def retranslateUi(self):
        self.interruptGroup.setTitle(translate('CueSettings', 'Interrupt Fade'))
        self.actionGroup.setTitle(translate('CueSettings', 'Fade Action'))

    def load_settings(self, settings):
        # Interrupt
        self.interruptFadeEdit.setDuration(
            float(settings['Cue'].get('interruptfade', 0)))
        self.interruptFadeEdit.setFadeType(
            settings['Cue'].get('interruptfadetype', ''))

        # FadeAction
        self.fadeActionEdit.setDuration(
            float(settings['Cue'].get('fadeactionduration', 0)))
        self.fadeActionEdit.setFadeType(
            settings['Cue'].get('fadeactiontype', ''))

    def get_settings(self):
        return {'Cue': {
            'interruptfade': str(self.interruptFadeEdit.duration()),
            'interruptfadetype': self.interruptFadeEdit.fadeType(),
            'fadeactionduration': str(self.fadeActionEdit.duration()),
            'fadeactiontype': self.fadeActionEdit.fadeType()
        }}



# -*- coding: utf-8 -*-
#
# This file is part of Linux Show Player
#
# Copyright 2012-2017 Francesco Ceruti <ceppofrancy@gmail.com>
#
# Linux Show Player is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Linux Show Player is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Linux Show Player.  If not, see <http://www.gnu.org/licenses/>.

from PyQt5.QtCore import QSize
from PyQt5.QtWidgets import QPushButton


class QIconPushButton(QPushButton):
    """QPushButton that resize dynamically it's icon."""

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.__icon_margin = 5

    def setIconMargin(self, margin):
        self.__icon_margin = int(margin)

    def iconMargin(self):
        return self.__icon_margin

    def resizeEvent(self, event):
        size = min(self.width(), self.height()) - 8
        self.setIconSize(QSize(size, size))



# -*- coding: utf-8 -*-
#
# This file is part of Linux Show Player
#
# Copyright 2012-2016 Francesco Ceruti <ceppofrancy@gmail.com>
#
# Linux Show Player is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Linux Show Player is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Linux Show Player.  If not, see <http://www.gnu.org/licenses/>.

from abc import abstractmethod

from PyQt5.QtWidgets import QAction, QMenu, qApp

from lisp.core.actions_handler import MainActionsHandler
from lisp.core.signal import Signal
from lisp.core.util import greatest_common_superclass
from lisp.cues.cue import Cue, CueAction
from lisp.cues.cue_actions import UpdateCueAction, \
    UpdateCuesAction
from lisp.layouts.cue_menu_registry import CueMenuRegistry
from lisp.ui.mainwindow import MainWindow
from lisp.ui.settings.cue_settings import CueSettings


class CueLayout:
    # Layout name
    NAME = 'Base'
    # Layout short description
    DESCRIPTION = 'No description'
    # Layout details (some useful info)
    DETAILS = ''

    cm_registry = CueMenuRegistry()

    def __init__(self, cue_model):
        self._cue_model = cue_model

        self.cue_executed = Signal()    # After a cue is executed
        self.focus_changed = Signal()   # After the focused cue is changed
        self.key_pressed = Signal()     # After a key is pressed

    @property
    def cue_model(self):
        """:rtype: lisp.core.cue_model.CueModel"""
        return self._cue_model

    @property
    @abstractmethod
    def model_adapter(self):
        """:rtype: lisp.core.model_adapter.ModelAdapter"""

    def current_cue(self):
        """Return the current cue, or None.

        :rtype: lisp.cues.cue.Cue
        """
        if self.model_adapter:
            try:
                return self.model_adapter.item(self.current_index())
            except IndexError:
                pass

    def current_index(self):
        """Return the current index, or -1."""
        return -1

    def set_current_cue(self, cue):
        """Set the current cue."""
        self.set_current_index(cue.index)

    def set_current_index(self, index):
        """Set the current cue by index"""

    def go(self, action=CueAction.Default, advance=1):
        """Execute the current cue and go ahead.

        .. Note::
            The advance value can be ignored by the layout.

        :param action: the action the cue should execute
        :param advance: number of index to advance (with negative will go back)
        :rtype: lisp.cues.cue.Cue
        """

    @abstractmethod
    def finalize(self):
        """Destroy all the layout elements"""

    def edit_cue(self, cue):
        edit_ui = CueSettings(cue, parent=MainWindow())

        def on_apply(settings):
            action = UpdateCueAction(settings, cue)
            MainActionsHandler.do_action(action)

        edit_ui.on_apply.connect(on_apply)
        edit_ui.exec_()

    def edit_selected_cues(self):
        cues = self.get_selected_cues()

        if cues:
            # Use the greatest common superclass between the selected cues
            edit_ui = CueSettings(cue_class=greatest_common_superclass(cues))

            def on_apply(settings):
                action = UpdateCuesAction(settings, cues)
                MainActionsHandler.do_action(action)

            edit_ui.on_apply.connect(on_apply)
            edit_ui.exec_()

    @abstractmethod
    def get_context_cue(self):
        """Return the last cue in the context-menu scope, or None"""
        return None

    @abstractmethod
    def get_selected_cues(self, cue_class=Cue):
        """Return a list of all selected cues, filtered by cue_class"""
        return []

    @abstractmethod
    def invert_selection(self):
        """Invert selection"""

    @abstractmethod
    def select_all(self, cue_class=Cue):
        """Select all the cues"""

    @abstractmethod
    def deselect_all(self, cue_class=Cue):
        """Deselect all the cues"""

    @staticmethod
    def show_context_menu(position):
        menu_edit = MainWindow().menuEdit
        menu_edit.move(position)
        menu_edit.show()

        # Adjust the menu position
        desktop = qApp.desktop().availableGeometry()
        menu_rect = menu_edit.geometry()

        if menu_rect.bottom() > desktop.bottom():
            menu_edit.move(menu_edit.x(), menu_edit.y() - menu_edit.height())
        if menu_rect.right() > desktop.right():
            menu_edit.move(menu_edit.x() - menu_edit.width(), menu_edit.y())

    def show_cue_context_menu(self, position):
        menu = QMenu(self)

        cue_class = self.get_context_cue().__class__
        for item in self.cm_registry.filter(cue_class):
            if isinstance(item, QAction):
                menu.addAction(item)
            elif isinstance(item, QMenu):
                menu.addMenu(item)

        menu.move(position)
        menu.show()

        # Adjust the menu position
        desktop = qApp.desktop().availableGeometry()
        menu_rect = menu.geometry()

        if menu_rect.bottom() > desktop.bottom():
            menu.move(menu.x(), menu.y() - menu.height())
        if menu_rect.right() > desktop.right():
            menu.move(menu.x() - menu.width(), menu.y())



# -*- coding: utf-8 -*-
#
# This file is part of Linux Show Player
#
# Copyright 2012-2016 Francesco Ceruti <ceppofrancy@gmail.com>
#
# Linux Show Player is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Linux Show Player is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Linux Show Player.  If not, see <http://www.gnu.org/licenses/>

from urllib.parse import unquote, quote

from lisp.backend.audio_utils import uri_duration
from lisp.modules.gst_backend.gi_repository import Gst, GstPbutils


def gst_uri_duration(uri):
    # First try to use the base implementation, because it's faster
    duration = uri_duration(uri)
    try:
        if duration <= 0:
            duration = gst_uri_metadata(uri).get_duration() // Gst.MSECOND
    finally:
        return duration if duration >= 0 else 0


def gst_mime_types():
    for gtf in Gst.TypeFindFactory.get_list():
        caps = gtf.get_caps()

        if caps is not None:
            for i in range(caps.get_size()):
                mime = caps.get_structure(i).to_string()
                extensions = gtf.get_extensions()
                yield mime, extensions


def gst_uri_metadata(uri):
    """Discover media-file metadata using GStreamer."""
    discoverer = GstPbutils.Discoverer()
    uri = uri.split("://")
    info = discoverer.discover_uri(uri[0] + "://" + quote(unquote(uri[1])))

    return info


# Adaption of the code found in https://github.com/ch3pjw/pyam
def gst_parse_tag_list(gst_tag_list):
    """Takes a GstTagList object and returns a dict."""
    parsed_tags = {}

    def parse_tag(gst_tag_list, tag_name, parsed_tags):
        parsed_tags[tag_name] = gst_tag_list.get_value_index(tag_name, 0)

    gst_tag_list.foreach(parse_tag, parsed_tags)
    return parsed_tags


# -*- coding: utf-8 -*-
#
# This file is part of Linux Show Player
#
# Copyright 2012-2016 Francesco Ceruti <ceppofrancy@gmail.com>
#
# Linux Show Player is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Linux Show Player is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Linux Show Player.  If not, see <http://www.gnu.org/licenses/>.

from PyQt5 import QtCore
from PyQt5.QtCore import QTime, QT_TRANSLATE_NOOP
from PyQt5.QtWidgets import QVBoxLayout, QGroupBox, QPushButton, QLabel, \
    QHBoxLayout, QTimeEdit

from lisp.application import Application
from lisp.core.has_properties import Property
from lisp.cues.cue import Cue, CueState
from lisp.cues.media_cue import MediaCue
from lisp.ui.cuelistdialog import CueSelectDialog
from lisp.ui.settings.cue_settings import CueSettingsRegistry
from lisp.ui.settings.settings_page import SettingsPage
from lisp.ui.ui_utils import translate


class SeekCue(Cue):
    Name = QT_TRANSLATE_NOOP('CueName', 'Seek Cue')

    target_id = Property()
    time = Property(default=-1)

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.name = translate('CueName', self.Name)

    def __start__(self, fade=False):
        cue = Application().cue_model.get(self.target_id)
        if isinstance(cue, MediaCue) and self.time >= 0:
            cue.media.seek(self.time)

        return False


class SeekCueSettings(SettingsPage):
    Name = QT_TRANSLATE_NOOP('SettingsPageName', 'Seek Settings')

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.setLayout(QVBoxLayout())
        self.layout().setAlignment(QtCore.Qt.AlignTop)

        self.cue_id = -1

        self.cueDialog = CueSelectDialog(
            cues=Application().cue_model.filter(MediaCue), parent=self)

        self.cueGroup = QGroupBox(self)
        self.cueGroup.setLayout(QVBoxLayout())
        self.layout().addWidget(self.cueGroup)

        self.cueButton = QPushButton(self.cueGroup)
        self.cueButton.clicked.connect(self.select_cue)
        self.cueGroup.layout().addWidget(self.cueButton)

        self.cueLabel = QLabel(self.cueGroup)
        self.cueLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.cueGroup.layout().addWidget(self.cueLabel)

        self.seekGroup = QGroupBox(self)
        self.seekGroup.setLayout(QHBoxLayout())
        self.layout().addWidget(self.seekGroup)

        self.seekEdit = QTimeEdit(self.seekGroup)
        self.seekEdit.setDisplayFormat('HH.mm.ss.zzz')
        self.seekGroup.layout().addWidget(self.seekEdit)

        self.seekLabel = QLabel(self.seekGroup)
        self.seekLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.seekGroup.layout().addWidget(self.seekLabel)

        self.retranslateUi()

    def retranslateUi(self):
        self.cueGroup.setTitle(translate('SeekCue', 'Cue'))
        self.cueButton.setText(translate('SeekCue', 'Click to select'))
        self.cueLabel.setText(translate('SeekCue', 'Not selected'))
        self.seekGroup.setTitle(translate('SeekCue', 'Seek'))
        self.seekLabel.setText(translate('SeekCue', 'Time to reach'))

    def select_cue(self):
        if self.cueDialog.exec_() == self.cueDialog.Accepted:
            cue = self.cueDialog.selected_cue()

            if cue is not None:
                self.cue_id = cue.id
                self.seekEdit.setMaximumTime(
                    QTime.fromMSecsSinceStartOfDay(cue.media.duration))
                self.cueLabel.setText(cue.name)

    def enable_check(self, enabled):
        self.cueGroup.setCheckable(enabled)
        self.cueGroup.setChecked(False)

        self.seekGroup.setCheckable(enabled)
        self.seekGroup.setChecked(False)

    def get_settings(self):
        return {'target_id': self.cue_id,
                'time': self.seekEdit.time().msecsSinceStartOfDay()}

    def load_settings(self, settings):
        if settings is not None:
            cue = Application().cue_model.get(settings.get('target_id'))
            if cue is not None:
                self.cue_id = settings['target_id']
                self.seekEdit.setMaximumTime(
                    QTime.fromMSecsSinceStartOfDay(cue.media.duration))
                self.cueLabel.setText(cue.name)

            self.seekEdit.setTime(
                QTime.fromMSecsSinceStartOfDay(settings.get('time', 0)))


CueSettingsRegistry().add_item(SeekCueSettings, SeekCue)



# -*- coding: utf-8 -*-
#
# This file is part of Linux Show Player
#
# Copyright 2012-2016 Francesco Ceruti <ceppofrancy@gmail.com>
#
# Linux Show Player is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Linux Show Player is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Linux Show Player.  If not, see <http://www.gnu.org/licenses/>.

from lisp.core.singleton import ABCSingleton


class Module(metaclass=ABCSingleton):
    """Interface for modules."""

    def terminate(self):
        """Finalize the module (e.g. free resources)."""



# -*- coding: utf-8 -*-
#
# This file is part of Linux Show Player
#
# Copyright 2012-2016 Francesco Ceruti <ceppofrancy@gmail.com>
#
# Linux Show Player is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Linux Show Player is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Linux Show Player.  If not, see <http://www.gnu.org/licenses/>.

from PyQt5.QtCore import pyqtSignal, QEvent
from PyQt5.QtWidgets import QLabel


class QClickLabel(QLabel):

    clicked = pyqtSignal(QEvent)

    def __init(self, parent):
        super().__init__(parent)

    def mouseReleaseEvent(self, e):
        if(self.contentsRect().contains(e.pos())):
            self.clicked.emit(e)



# -*- coding: utf-8 -*-
#
# This file is part of Linux Show Player
#
# Copyright 2012-2016 Francesco Ceruti <ceppofrancy@gmail.com>
# Copyright 2016 Thomas Achtner <info@offtools.de>
#
# Linux Show Player is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Linux Show Player is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Linux Show Player.  If not, see <http://www.gnu.org/licenses/>.

from PyQt5.QtCore import QT_TRANSLATE_NOOP, Qt
from PyQt5.QtWidgets import QGridLayout
from PyQt5.QtWidgets import QMessageBox
from PyQt5.QtWidgets import QVBoxLayout, QGroupBox, QLabel,\
    QCheckBox, QComboBox, QHBoxLayout, QSpinBox
from ola.OlaClient import OLADNotRunningException, OlaClient

from lisp.ui.mainwindow import MainWindow
from lisp.ui.settings.settings_page import CueSettingsPage,\
    SettingsPage
from lisp.ui.ui_utils import translate


class TimecodeSettings(SettingsPage):
    Name = QT_TRANSLATE_NOOP('SettingsPageName', 'Timecode Settings')

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.setLayout(QVBoxLayout())
        self.layout().setAlignment(Qt.AlignTop)

        self.groupBox = QGroupBox(self)
        self.groupBox.setLayout(QGridLayout())
        self.layout().addWidget(self.groupBox)

        self.activateBox = QCheckBox(self.groupBox)
        self.groupBox.layout().addWidget(self.activateBox, 0, 0)

        self.hresBox = QCheckBox(self.groupBox)
        self.groupBox.layout().addWidget(self.hresBox, 1, 0)

        self.formatLabel = QLabel(self.groupBox)
        self.groupBox.layout().addWidget(self.formatLabel, 2, 0)

        self.formatBox = QComboBox(self.groupBox)
        self.formatBox.addItem('FILM')
        self.formatBox.addItem('EBU')
        self.formatBox.addItem('SMPTE')
        self.groupBox.layout().addWidget(self.formatBox, 2, 1)

        self.retranslateUi()

    def retranslateUi(self):
        self.groupBox.setTitle(
            translate('TimecodeSettings', 'OLA Timecode Settings'))
        self.activateBox.setText(translate('TimecodeSettings', 'Enable Plugin'))
        self.hresBox.setText(
            translate('TimecodeSettings', 'High-Resolution Timecode'))
        self.formatLabel.setText(
            translate('TimecodeSettings', 'Timecode Format:'))

    def testOla(self):
        if self.activateBox.isChecked():
            try:
                client = OlaClient()
                del client
            except OLADNotRunningException:
                QMessageBox.warning(
                    MainWindow(),
                    translate('TimecodeSettings', 'OLA status'),
                    translate('TimecodeSettings',
                              'OLA is not running - start the OLA daemon.')
                )

    def get_settings(self):
        return {'Timecode': {
            'enabled': str(self.activateBox.isChecked()),
            'hres': str(self.hresBox.isChecked()),
            'format': self.formatBox.currentText()
        }}

    def load_settings(self, settings):
        settings = settings.get('Timecode', {})

        self.activateBox.setChecked(settings.get('enabled') == 'True')
        self.hresBox.setChecked(settings.get('hres') == 'True')
        self.formatBox.setCurrentText(settings.get('format', ''))

        self.activateBox.stateChanged.connect(self.testOla)


class TimecodeCueSettings(CueSettingsPage):
    Name = QT_TRANSLATE_NOOP('SettingsPageName', 'Timecode')

    def __init__(self, cue_class, **kwargs):
        super().__init__(cue_class, **kwargs)
        self.setLayout(QVBoxLayout())
        self.layout().setAlignment(Qt.AlignTop)

        self.groupBox = QGroupBox(self)
        self.groupBox.setLayout(QGridLayout())
        self.layout().addWidget(self.groupBox)

        # enable / disable timecode
        self.enableCheck = QCheckBox(self.groupBox)
        self.enableCheck.setChecked(False)
        self.groupBox.layout().addWidget(self.enableCheck, 0, 0)

        # Hours can be replaced by cue number h:m:s:frames -> CUE:m:s:frames
        self.useHoursCheck = QCheckBox(self.groupBox)
        self.useHoursCheck.setChecked(True)
        self.groupBox.layout().addWidget(self.useHoursCheck, 1, 0)

        self.trackSpin = QSpinBox(self)
        self.trackSpin.setMinimum(0)
        self.trackSpin.setMaximum(99)
        self.useHoursCheck.stateChanged.connect(self.trackSpin.setEnabled)
        self.groupBox.layout().addWidget(self.trackSpin, 2, 0)

        self.trackLabel = QLabel(self.groupBox)
        self.trackLabel.setAlignment(Qt.AlignCenter)
        self.groupBox.layout().addWidget(self.trackLabel, 2, 1)

        self.layout().addSpacing(50)

        self.warnLabel = QLabel(self)
        self.warnLabel.setAlignment(Qt.AlignCenter)
        self.warnLabel.setStyleSheet('color: #FFA500; font-weight: bold')
        self.layout().addWidget(self.warnLabel)

        self.retranslateUi()

    def retranslateUi(self):
        self.groupBox.setTitle('Timecode')
        self.useHoursCheck.setText(
            translate('TimecodeSettings',
                      'Replace HOURS by a static track number'))
        self.enableCheck.setText(
            translate('TimecodeSettings', 'Enable ArtNet Timecode'))
        self.trackLabel.setText(
            translate('TimecodeSettings', 'Track number'))
        self.warnLabel.setText(
            translate('TimecodeSettings',
                      'To send ArtNet Timecode you need to setup a running OLA'
                      ' session!'))

    def get_settings(self):
        settings = {
            'enabled': self.enableCheck.isChecked(),
            'replace_hours': self.useHoursCheck.isChecked(),
            'track': self.trackSpin.value()
        }

        return {'timecode': settings}

    def load_settings(self, settings):
        settings = settings.get('timecode', {})
        self.enableCheck.setChecked(settings.get('enabled', False))
        self.useHoursCheck.setChecked(settings.get('replace_hours', False))
        self.trackSpin.setValue(settings.get('track', 0))



# -*- coding: utf-8 -*-
"""
Linux Show Player

Cue player for live shows
"""

__author__ = 'Francesco Ceruti'
__email__ = 'ceppofrancy@gmail.com'
__url__ = 'https://github.com/FrancescoCeruti/linux-show-player'
__license__ = 'GPLv3'
__version__ = '0.5'



# -*- coding: utf-8 -*-
#
# This file is part of Linux Show Player
#
# Copyright 2012-2016 Francesco Ceruti <ceppofrancy@gmail.com>
#
# Linux Show Player is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Linux Show Player is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Linux Show Player.  If not, see <http://www.gnu.org/licenses/>.

from PyQt5.QtCore import Qt, QT_TRANSLATE_NOOP
from PyQt5.QtWidgets import QGroupBox, QVBoxLayout, QCheckBox, QComboBox

from lisp import layouts
from lisp.ui.styles import styles
from lisp.ui.settings.settings_page import SettingsPage
from lisp.ui.ui_utils import translate


class AppGeneral(SettingsPage):
    Name = QT_TRANSLATE_NOOP('SettingsPageName', 'General')

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.setLayout(QVBoxLayout())
        self.layout().setAlignment(Qt.AlignTop)

        # Startup layout
        self.layoutGroup = QGroupBox(self)
        self.layoutGroup.setTitle(
            translate('AppGeneralSettings', 'Startup layout'))
        self.layoutGroup.setLayout(QVBoxLayout())
        self.layout().addWidget(self.layoutGroup)

        self.startupDialogCheck = QCheckBox(self.layoutGroup)
        self.startupDialogCheck.setText(
            translate('AppGeneralSettings', 'Use startup dialog'))
        self.layoutGroup.layout().addWidget(self.startupDialogCheck)

        self.layoutCombo = QComboBox(self.layoutGroup)
        self.layoutCombo.addItems([lay.NAME for lay in layouts.get_layouts()])
        self.layoutGroup.layout().addWidget(self.layoutCombo)

        self.startupDialogCheck.clicked.connect(
            lambda check: self.layoutCombo.setEnabled(not check))

        # Application style
        self.themeGroup = QGroupBox(self)
        self.themeGroup.setTitle(
            translate('AppGeneralSettings', 'Application theme'))
        self.themeGroup.setLayout(QVBoxLayout())
        self.layout().addWidget(self.themeGroup)

        self.themeCombo = QComboBox(self.themeGroup)
        self.themeCombo.addItems(styles.styles())
        self.themeGroup.layout().addWidget(self.themeCombo)

    def get_settings(self):
        conf = {'Layout': {}, 'Theme': {}}

        if self.startupDialogCheck.isChecked():
            conf['Layout']['default'] = 'NoDefault'
        else:
            conf['Layout']['default'] = self.layoutCombo.currentText()

        conf['Theme']['theme'] = self.themeCombo.currentText()
        styles.apply_style(self.themeCombo.currentText())

        return conf

    def load_settings(self, settings):
        if 'default' in settings['Layout']:
            if settings['Layout']['default'].lower() == 'nodefault':
                self.startupDialogCheck.setChecked(True)
                self.layoutCombo.setEnabled(False)
            else:
                self.layoutCombo.setCurrentText(settings['Layout']['default'])
        if 'theme' in settings['Theme']:
            self.themeCombo.setCurrentText(settings['Theme']['theme'])



# -*- coding: utf-8 -*-
#
# This file is part of Linux Show Player
#
# Copyright 2012-2016 Francesco Ceruti <ceppofrancy@gmail.com>
#
# Linux Show Player is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Linux Show Player is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Linux Show Player.  If not, see <http://www.gnu.org/licenses/>.

from PyQt5.QtCore import Qt, QT_TRANSLATE_NOOP
from PyQt5.QtWidgets import QVBoxLayout, QGroupBox, QHBoxLayout, QTextEdit, \
    QSpinBox, QLabel, QLineEdit

from lisp.ui.settings.settings_page import SettingsPage
from lisp.ui.widgets import QColorButton
from lisp.ui.ui_utils import translate


class Appearance(SettingsPage):
    Name = QT_TRANSLATE_NOOP('SettingsPageName', 'Appearance')

    def __init__(self, **kwargs):
        super().__init__()
        self.setLayout(QVBoxLayout())

        # Name
        self.cueNameGroup = QGroupBox(self)
        self.cueNameGroup.setLayout(QHBoxLayout())
        self.layout().addWidget(self.cueNameGroup)

        self.cueNameEdit = QLineEdit(self.cueNameGroup)
        self.cueNameGroup.layout().addWidget(self.cueNameEdit)

        # Description
        self.cueDescriptionGroup = QGroupBox(self)
        self.cueDescriptionGroup.setLayout(QHBoxLayout())
        self.layout().addWidget(self.cueDescriptionGroup)

        self.cueDescriptionEdit = QTextEdit(self.cueNameGroup)
        self.cueDescriptionGroup.layout().addWidget(self.cueDescriptionEdit)

        # Font
        self.fontSizeGroup = QGroupBox(self)
        self.fontSizeGroup.setLayout(QHBoxLayout())
        self.layout().addWidget(self.fontSizeGroup)

        self.fontSizeSpin = QSpinBox(self.fontSizeGroup)
        self.fontSizeSpin.setValue(QLabel().fontInfo().pointSize())
        self.fontSizeGroup.layout().addWidget(self.fontSizeSpin)

        # Color
        self.colorGroup = QGroupBox(self)
        self.colorGroup.setLayout(QHBoxLayout())
        self.layout().addWidget(self.colorGroup)

        self.colorBButton = QColorButton(self.colorGroup)
        self.colorFButton = QColorButton(self.colorGroup)

        self.colorGroup.layout().addWidget(self.colorBButton)
        self.colorGroup.layout().addWidget(self.colorFButton)

        # Warning
        self.warning = QLabel(self)
        self.warning.setText(translate('CueAppearanceSettings',
                                       'The appearance depends on the layout'))
        self.warning.setAlignment(Qt.AlignCenter)
        self.warning.setStyleSheet("color: #FFA500; font-weight: bold")
        self.layout().addWidget(self.warning)

        self.retranslateUi()

    def retranslateUi(self):
        self.cueNameGroup.setTitle(
            translate('CueAppearanceSettings', 'Cue name'))
        self.cueNameEdit.setText(translate('CueAppearanceSettings',
                                           'NoName'))
        self.cueDescriptionGroup.setTitle(
            translate('CueAppearanceSettings', 'Description/Note'))
        self.fontSizeGroup.setTitle(
            translate('CueAppearanceSettings', 'Set Font Size'))
        self.colorGroup.setTitle(translate('CueAppearanceSettings', 'Color'))
        self.colorBButton.setText(
            translate('CueAppearanceSettings', 'Select background color'))
        self.colorFButton.setText(
            translate('CueAppearanceSettings', 'Select font color'))

    def enable_check(self, enable):
        self.cueNameGroup.setCheckable(enable)
        self.cueNameGroup.setChecked(False)

        self.cueDescriptionGroup.setChecked(enable)
        self.cueDescriptionGroup.setChecked(False)

        self.fontSizeGroup.setCheckable(enable)
        self.fontSizeGroup.setChecked(False)

        self.colorGroup.setCheckable(enable)
        self.colorGroup.setChecked(False)

    def get_settings(self):
        conf = {}
        style = {}
        checkable = self.cueNameGroup.isCheckable()

        if not (checkable and not self.cueNameGroup.isChecked()):
            conf['name'] = self.cueNameEdit.text()
        if not (checkable and not self.cueDescriptionGroup.isChecked()):
            conf['description'] = self.cueDescriptionEdit.toPlainText()
        if not (checkable and not self.colorGroup.isChecked()):
            if self.colorBButton.color() is not None:
                style['background'] = self.colorBButton.color()
            if self.colorFButton.color() is not None:
                style['color'] = self.colorFButton.color()
        if not (checkable and not self.fontSizeGroup.isChecked()):
            style['font-size'] = str(self.fontSizeSpin.value()) + 'pt'

        if style:
            conf['stylesheet'] = dict_to_css(style)

        return conf

    def load_settings(self, settings):
        if 'name' in settings:
            self.cueNameEdit.setText(settings['name'])
        if 'description' in settings:
            self.cueDescriptionEdit.setText(settings['description'])
        if 'stylesheet' in settings:
            settings = css_to_dict(settings['stylesheet'])
            if 'background' in settings:
                self.colorBButton.setColor(settings['background'])
            if 'color' in settings:
                self.colorFButton.setColor(settings['color'])
            if 'font-size' in settings:
                # [:-2] for removing "pt"
                self.fontSizeSpin.setValue(int(settings['font-size'][:-2]))


def css_to_dict(css):
    dict = {}
    css = css.strip()

    for attribute in css.split(';'):
        try:
            name, value = attribute.split(':')
            dict[name.strip()] = value.strip()
        except:
            pass

    return dict


def dict_to_css(css_dict):
    css = ''
    for name in css_dict:
        css += name + ':' + str(css_dict[name]) + ';'

    return css



# -*- coding: utf-8 -*-
#
# This file is part of Linux Show Player
#
# Copyright 2012-2016 Francesco Ceruti <ceppofrancy@gmail.com>
#
# Linux Show Player is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Linux Show Player is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Linux Show Player.  If not, see <http://www.gnu.org/licenses/>.

from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QListWidget, QListWidgetItem

from lisp.core.signal import Connection
from lisp.layouts.list_layout.playing_mediawidget import get_running_widget


class RunningCuesListWidget(QListWidget):

    def __init__(self, running_model, **kwargs):
        super().__init__(**kwargs)
        self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.setFocusPolicy(Qt.NoFocus)
        self.setSelectionMode(self.NoSelection)

        self._running_cues = {}
        self._running_model = running_model
        self._running_model.item_added.connect(
            self._item_added, Connection.QtQueued)
        self._running_model.item_removed.connect(
            self._item_removed, Connection.QtQueued)
        self._running_model.model_reset.connect(
            self._model_reset, Connection.QtQueued)

        self.__dbmeter_visible = False
        self.__seek_visible = False
        self.__accurate_time = False

    @property
    def dbmeter_visible(self):
        return self.__dbmeter_visible

    @dbmeter_visible.setter
    def dbmeter_visible(self, visible):
        self.__dbmeter_visible = visible
        for item in self._running_cues.values():
            try:
                self.itemWidget(item).set_dbmeter_visible(visible)
            except AttributeError:
                pass

    @property
    def seek_visible(self):
        return self.__seek_visible

    @seek_visible.setter
    def seek_visible(self, visible):
        self.__seek_visible = visible
        for item in self._running_cues.values():
            try:
                self.itemWidget(item).set_seek_visible(visible)
            except AttributeError:
                pass

    @property
    def accurate_time(self):
        return self.__accurate_time

    @accurate_time.setter
    def accurate_time(self, accurate):
        self.__accurate_time = accurate
        for item in self._running_cues.values():
            self.itemWidget(item).set_accurate_time(accurate)

    def _item_added(self, cue):
        widget = get_running_widget(cue, parent=self)
        widget.set_accurate_time(self.__accurate_time)
        try:
            widget.set_dbmeter_visible(self.__dbmeter_visible)
            widget.set_seek_visible(self.__seek_visible)
        except AttributeError:
            pass

        item = QListWidgetItem()
        item.setSizeHint(widget.size())

        self.addItem(item)
        self.setItemWidget(item, widget)
        self._running_cues[cue] = item

    def _item_removed(self, cue):
        item = self._running_cues.pop(cue)
        widget = self.itemWidget(item)
        row = self.indexFromItem(item).row()

        self.removeItemWidget(item)
        self.takeItem(row)

        widget.deleteLater()

    def _model_reset(self):
        for cue in list(self._running_cues.keys()):
            self._item_removed(cue)



# -*- coding: utf-8 -*-
#
# This file is part of Linux Show Player
#
# Copyright 2012-2016 Francesco Ceruti <ceppofrancy@gmail.com>
#
# Linux Show Player is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Linux Show Player is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Linux Show Player.  If not, see <http://www.gnu.org/licenses/>.

from abc import abstractmethod


class Action:
    """Base class for actions.

    Action provides the ability to revert the changes done in the "do" method,
    via "undo" method, and redo them via the "redo" method.
    An action could provide, via the "log" function, a simple log message.

    .. warning::
        Actions may reference external objects, preventing gc.
    """

    __slots__ = ()

    @abstractmethod
    def do(self):
        """Do something"""

    @abstractmethod
    def undo(self):
        """Revert what do function has done"""

    def redo(self):
        """Redo after reverting

        The default implementation call the "do" function.
        """
        self.do()

    def log(self):
        """Used for logging

        :return: A log message
        :rtype: str
        """
        return ''



# -*- coding: utf-8 -*-
#
# This file is part of Linux Show Player
#
# Copyright 2012-2017 Francesco Ceruti <ceppofrancy@gmail.com>
#
# Linux Show Player is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Linux Show Player is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Linux Show Player.  If not, see <http://www.gnu.org/licenses/>.

from PyQt5.QtCore import QTime, Qt
from PyQt5.QtWidgets import QGroupBox, QHBoxLayout, QTimeEdit, QLabel, \
    QSpinBox, QVBoxLayout

from lisp.ui.settings.settings_page import SettingsPage
from lisp.ui.ui_utils import translate


class MediaCueSettings(SettingsPage):
    Name = 'Media-Cue'

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.setLayout(QVBoxLayout(self))

        # Start time
        self.startGroup = QGroupBox(self)
        self.startGroup.setLayout(QHBoxLayout())
        self.layout().addWidget(self.startGroup)

        self.startEdit = QTimeEdit(self.startGroup)
        self.startEdit.setDisplayFormat('HH.mm.ss.zzz')
        self.startGroup.layout().addWidget(self.startEdit)

        self.startLabel = QLabel(self.startGroup)
        self.startLabel.setAlignment(Qt.AlignCenter)
        self.startGroup.layout().addWidget(self.startLabel)

        # Stop time
        self.stopGroup = QGroupBox(self)
        self.stopGroup.setLayout(QHBoxLayout())
        self.layout().addWidget(self.stopGroup)

        self.stopEdit = QTimeEdit(self.stopGroup)
        self.stopEdit.setDisplayFormat('HH.mm.ss.zzz')
        self.stopGroup.layout().addWidget(self.stopEdit)

        self.stopLabel = QLabel(self.stopGroup)
        self.stopLabel.setAlignment(Qt.AlignCenter)
        self.stopGroup.layout().addWidget(self.stopLabel)

        # Loop
        self.loopGroup = QGroupBox(self)
        self.loopGroup.setLayout(QHBoxLayout())
        self.layout().addWidget(self.loopGroup)

        self.spinLoop = QSpinBox(self.loopGroup)
        self.spinLoop.setRange(-1, 1000000)
        self.loopGroup.layout().addWidget(self.spinLoop)

        self.loopLabel = QLabel(self.loopGroup)
        self.loopLabel.setAlignment(Qt.AlignCenter)
        self.loopGroup.layout().addWidget(self.loopLabel)

        self.retranslateUi()

    def retranslateUi(self):
        self.startGroup.setTitle(translate('MediaCueSettings', 'Start time'))
        self.stopLabel.setText(
            translate('MediaCueSettings', 'Stop position of the media'))
        self.stopGroup.setTitle(translate('MediaCueSettings', 'Stop time'))
        self.startLabel.setText(
            translate('MediaCueSettings', 'Start position of the media'))
        self.loopGroup.setTitle(translate('MediaCueSettings', 'Loop'))
        self.loopLabel.setText(
            translate('MediaCueSettings', 'Repetition after first play '
                                          '(-1 = infinite)'))

    def get_settings(self):
        conf = {'_media_': {}}
        checkable = self.startGroup.isCheckable()

        if not (checkable and not self.startGroup.isChecked()):
            time = self.startEdit.time().msecsSinceStartOfDay()
            conf['_media_']['start_time'] = time
        if not (checkable and not self.stopGroup.isChecked()):
            time = self.stopEdit.time().msecsSinceStartOfDay()
            conf['_media_']['stop_time'] = time
        if not (checkable and not self.loopGroup.isChecked()):
            conf['_media_']['loop'] = self.spinLoop.value()

        return conf

    def enable_check(self, enable):
        self.startGroup.setCheckable(enable)
        self.startGroup.setChecked(False)

        self.stopGroup.setCheckable(enable)
        self.stopGroup.setChecked(False)

        self.loopGroup.setCheckable(enable)
        self.loopGroup.setChecked(False)

    def load_settings(self, settings):
        if '_media_' in settings:
            if 'loop' in settings['_media_']:
                self.spinLoop.setValue(settings['_media_']['loop'])
            if 'start_time' in settings['_media_']:
                t = self._to_qtime(settings['_media_']['start_time'])
                self.startEdit.setTime(t)
            if 'stop_time' in settings['_media_']:
                t = self._to_qtime(settings['_media_']['stop_time'])
                self.stopEdit.setTime(t)

            t = self._to_qtime(settings['_media_'].get('duration', 0))
            self.startEdit.setMaximumTime(t)
            self.stopEdit.setMaximumTime(t)

    def _to_qtime(self, m_seconds):
        return QTime.fromMSecsSinceStartOfDay(m_seconds)



# -*- coding: utf-8 -*-
#
# This file is part of Linux Show Player
#
# Copyright 2012-2017 Francesco Ceruti <ceppofrancy@gmail.com>
#
# Linux Show Player is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Linux Show Player is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Linux Show Player.  If not, see <http://www.gnu.org/licenses/>.

from threading import Lock
from uuid import uuid4

from lisp.core.configuration import config
from lisp.core.decorators import async
from lisp.core.fade_functions import FadeInType, FadeOutType
from lisp.core.has_properties import HasProperties, Property, WriteOnceProperty
from lisp.core.rwait import RWait
from lisp.core.signal import Signal
from lisp.core.util import EqEnum


class CueState:
    Invalid = 0

    Error = 1
    Stop = 2
    Running = 4
    Pause = 8

    PreWait = 16
    PostWait = 32
    PreWait_Pause = 64
    PostWait_Pause = 128

    IsRunning = Running | PreWait | PostWait
    IsPaused = Pause | PreWait_Pause | PostWait_Pause
    IsStopped = Error | Stop


class CueAction(EqEnum):
    Default = 'Default'
    FadeIn = 'FadeIn'
    FadeOut = 'FadeOut'
    FadeInStart = 'FadeInStart'
    FadeOutStop = 'FadeOutStop'
    FadeOutPause = 'FadeOutPause'
    FadeOutInterrupt = 'FadeOutInterrupt'
    Interrupt = 'Interrupt'
    Start = 'Start'
    Stop = 'Stop'
    Pause = 'Pause'


class CueNextAction(EqEnum):
    DoNothing = 'DoNothing'
    AutoNext = 'AutoNext'
    AutoFollow = 'AutoFollow'


class Cue(HasProperties):
    """Cue(s) are the base component for implement any kind of live-controllable
    element (live = during a show).

    A cue implement his behavior(s) reimplementing __start__, __stop__,
    __pause__ and __interrupt__ methods.

    Cue provide **(and any subclass should do the same)** properties via
    HasProperties/Property specifications.

    :ivar _type_: Cue type (class name). Should NEVER change after init.
    :ivar id: Identify the cue uniquely. Should NEVER change after init.
    :ivar index: Cue position in the view.
    :ivar name: Cue visualized name.
    :ivar description: Cue text description.
    :ivar stylesheet: Cue style, used by the view.
    :ivar duration: The cue duration in milliseconds. (0 means no duration)
    :ivar pre_wait: Cue pre-wait in seconds.
    :ivar post_wait: Cue post-wait in seconds.
    :ivar next_action: What do after post_wait.
    :ivar fadein_type: Fade-In type
    :ivar fadeout_type: Fade-Out type
    :ivar fadein_duration: Fade-In duration in seconds
    :ivar fadeout_duration: Fade-Out duration in seconds
    :ivar default_start_action: action to execute to start
    :ivar default_stop_action: action to execute to stop
    :cvar CueActions: actions supported by the cue (default: CueAction.Start)

    A cue should declare CueAction.Default as supported only if CueAction.Start
    and CueAction.Stop are both supported.
    If CueAction.Stop is supported, CueAction.Interrupt should be supported.

    .. Note::
        If 'next_action' is AutoFollow or DoNothing, the postwait is not
        performed.
    """
    Name = 'Cue'

    _type_ = WriteOnceProperty()
    id = WriteOnceProperty()
    name = Property(default='Untitled')
    index = Property(default=-1)
    description = Property(default='')
    stylesheet = Property(default='')
    duration = Property(default=0)
    pre_wait = Property(default=0)
    post_wait = Property(default=0)
    next_action = Property(default=CueNextAction.DoNothing.value)
    fadein_type = Property(default=FadeInType.Linear.name)
    fadeout_type = Property(default=FadeOutType.Linear.name)
    fadein_duration = Property(default=0)
    fadeout_duration = Property(default=0)
    default_start_action = Property(default=CueAction.Start.value)
    default_stop_action = Property(default=CueAction.Stop.value)

    CueActions = (CueAction.Start,)

    def __init__(self, id=None):
        super().__init__()
        self.id = str(uuid4()) if id is None else id
        self._type_ = self.__class__.__name__

        self._st_lock = Lock()
        self._state = CueState.Stop
        self._prewait = RWait()
        self._postwait = RWait()

        # Pre-Wait signals
        self.prewait_start = self._prewait.start
        self.prewait_ended = self._prewait.ended
        self.prewait_paused = self._prewait.paused
        self.prewait_stopped = self._prewait.stopped

        # Post-Wait signals
        self.postwait_start = self._postwait.start
        self.postwait_ended = self._postwait.ended
        self.postwait_paused = self._postwait.paused
        self.postwait_stopped = self._postwait.stopped

        # Fade signals
        self.fadein_start = Signal()
        self.fadein_end = Signal()
        self.fadeout_start = Signal()
        self.fadeout_end = Signal()

        # Status signals
        self.interrupted = Signal()     # self
        self.started = Signal()         # self
        self.stopped = Signal()         # self
        self.paused = Signal()          # self
        self.error = Signal()           # self, error, details
        self.next = Signal()            # self
        self.end = Signal()             # self

        self.changed('next_action').connect(self.__next_action_changed)

    def execute(self, action=CueAction.Default):
        """Execute the specified action, if supported.

        .. Note::
            Even if not specified in Cue.CueActions, when CueAction.Default
            is given, a "default" action is selected depending on the current
            cue state, if this action is not supported nothing will be done.

        :param action: the action to be performed
        :type action: CueAction
        """
        if action == CueAction.Default:
            if self._state & CueState.IsRunning:
                action = CueAction(self.default_stop_action)
            else:
                action = CueAction(self.default_start_action)

        if action is CueAction.Interrupt:
            self.interrupt()
        elif action is CueAction.FadeOutInterrupt:
            self.interrupt(fade=True)
        elif action in self.CueActions:
            if action == CueAction.Start:
                self.start()
            elif action == CueAction.FadeInStart:
                self.start(fade=self.fadein_duration > 0)
            elif action == CueAction.Stop:
                self.stop()
            elif action == CueAction.FadeOutStop:
                self.stop(fade=self.fadeout_duration > 0)
            elif action == CueAction.Pause:
                self.pause()
            elif action == CueAction.FadeOutPause:
                self.pause(fade=self.fadeout_duration > 0)
            elif action == CueAction.FadeOut:
                duration = config['Cue'].getfloat('FadeActionDuration')
                fade_type = FadeOutType[config['Cue'].get('FadeActionType')]
                self.fadeout(duration, fade_type)
            elif action == CueAction.FadeIn:
                duration = config['Cue'].getfloat('FadeActionDuration')
                fade_type = FadeInType[config['Cue'].get('FadeActionType')]
                self.fadein(duration, fade_type)

    @async
    def start(self, fade=False):
        """Start the cue."""

        # If possible acquire the state-lock, otherwise return
        if not self._st_lock.acquire(blocking=False):
            return

        try:
            # If we are already running release and return
            if self._state & CueState.IsRunning:
                return

            state = self._state

            # PreWait
            if self.pre_wait and state & (CueState.IsStopped |
                                          CueState.PreWait_Pause):
                self._state = CueState.PreWait
                # Start the wait, the lock is released during the wait and
                # re-acquired after
                if not self._prewait.wait(self.pre_wait, lock=self._st_lock):
                    # PreWait interrupted, check the state to be correct
                    if self._state & CueState.PreWait:
                        self._state ^= CueState.PreWait
                    return

            # Cue-Start (still locked), the __start__ function should not block
            if state & (CueState.IsStopped |
                        CueState.Pause |
                        CueState.PreWait_Pause):

                running = self.__start__(fade)
                self._state = CueState.Running
                self.started.emit(self)

                if not running:
                    self._ended()

            # PostWait (still locked)
            if state & (CueState.IsStopped |
                        CueState.PreWait_Pause |
                        CueState.PostWait_Pause):
                if self.next_action == CueNextAction.AutoNext:
                    self._state |= CueState.PostWait

                    if self._postwait.wait(self.post_wait, lock=self._st_lock):
                        # PostWait ended
                        self._state ^= CueState.PostWait
                        self.next.emit(self)
                    elif self._state & CueState.PostWait:
                        # PostWait interrupted, check the state to be correct
                        self._state ^= CueState.PostWait

                    # If the cue was only post-waiting we remain with
                    # an invalid state
                    if not self._state:
                        self._state = CueState.Stop
        finally:
            self._st_lock.release()

    def restart(self, fade=False):
        """Restart the cue if paused."""
        if self._state & CueState.IsPaused:
            self.start(fade)

    def __start__(self, fade=False):
        """Implement the cue `start` behavior.

        Long running task should not block this function (i.e. the fade should
        be performed in another thread).

        When called from `Cue.start()`, `_st_lock` is acquired.

        If the execution is instantaneous should return False, otherwise
        return True and call the `_ended` function later.

        :param fade: True if a fade should be performed (when supported)
        :type fade: bool
        :return: False if the cue is already terminated, True otherwise
            (e.g. asynchronous execution)
        :rtype: bool
        """
        return False

    @async
    def stop(self, fade=False):
        """Stop the cue."""

        # If possible acquire the state-lock, otherwise return
        if not self._st_lock.acquire(blocking=False):
            return

        try:
            # Stop PreWait (if in PreWait(_Pause) nothing else is "running")
            if self._state & (CueState.PreWait | CueState.PreWait_Pause):
                self._state = CueState.Stop
                self._prewait.stop()
            else:
                # Stop PostWait
                if self._state & (CueState.PostWait | CueState.PostWait_Pause):
                    # Remove PostWait or PostWait_Pause state
                    self._state = (
                        (self._state ^ CueState.PostWait) &
                        (self._state ^ CueState.PostWait_Pause)
                    )
                    self._postwait.stop()

                # Stop the cue
                if self._state & (CueState.Running | CueState.Pause):
                    # Here the __stop__ function should release and re-acquire
                    # the state-lock during a fade operation
                    if not self.__stop__(fade):
                        # Stop operation interrupted
                        return

                    # Remove Running or Pause state
                    self._state = (
                        (self._state ^ CueState.Running) &
                        (self._state ^ CueState.Pause)
                    )
                    self._state |= CueState.Stop
                    self.stopped.emit(self)
        finally:
            self._st_lock.release()

    def __stop__(self, fade=False):
        """Implement the cue `stop` behavior.

        Long running task should block this function (i.e. the fade should
        "block" this function), when this happen `_st_lock` must be released and
        then re-acquired.

        If called during a `fadeout` operation this should be interrupted,
        the cue stopped and return `True`.

        :param fade: True if a fade should be performed (when supported)
        :type fade: bool
        :return: False if interrupted, True otherwise
        :rtype: bool
        """
        return False

    @async
    def pause(self, fade=False):
        """Pause the cue."""

        # If possible acquire the state-lock, otherwise return
        if not self._st_lock.acquire(blocking=False):
            return

        try:
            # Pause PreWait (if in PreWait nothing else is "running")
            if self._state & CueState.PreWait:
                self._state ^= CueState.PreWait
                self._state |= CueState.PreWait_Pause
                self._prewait.pause()
            else:
                # Pause PostWait
                if self._state & CueState.PostWait:
                    self._state ^= CueState.PostWait
                    self._state |= CueState.PostWait_Pause
                    self._postwait.pause()

                # Pause the cue
                if self._state & CueState.Running:
                    # Here the __pause__ function should release and re-acquire
                    # the state-lock during a fade operation
                    if not self.__pause__(fade):
                        return

                    self._state ^= CueState.Running
                    self._state |= CueState.Pause
                    self.paused.emit(self)
        finally:
            self._st_lock.release()

    def __pause__(self, fade=False):
        """Implement the cue `pause` behavior.

        Long running task should block this function (i.e. the fade should
        "block" this function), when this happen `_st_lock` must be released and
        then re-acquired.

        If called during a `fadeout` operation this should be interrupted,
        the cue paused and return `True`.

        If during the execution the fade operation is interrupted this function
        must return `False`.

        :param fade: True if a fade should be performed (when supported)
        :type fade: bool
        :return: False if interrupted, True otherwise
        :rtype: bool
        """
        return False

    @async
    def interrupt(self, fade=False):
        """Interrupt the cue.

        :param fade: True if a fade should be performed (when supported)
        :type fade: bool
        """
        with self._st_lock:
            # Stop PreWait (if in PreWait(_Pause) nothing else is "running")
            if self._state & (CueState.PreWait | CueState.PreWait_Pause):
                self._state = CueState.Stop
                self._prewait.stop()
            else:
                # Stop PostWait
                if self._state & (CueState.PostWait | CueState.PostWait_Pause):
                    # Remove PostWait or PostWait_Pause state
                    self._state = (
                        (self._state ^ CueState.PostWait) &
                        (self._state ^ CueState.PostWait_Pause)
                    )
                    self._postwait.stop()

                # Interrupt the cue
                if self._state & (CueState.Running | CueState.Pause):
                    self.__interrupt__(fade)

                    # Remove Running or Pause state
                    self._state = (
                        (self._state ^ CueState.Running) &
                        (self._state ^ CueState.Pause)
                    )
                    self._state |= CueState.Stop
                    self.interrupted.emit(self)

    def __interrupt__(self, fade=False):
        """Implement the cue `interrupt` behavior.

        Long running task should block this function without releasing
        `_st_lock`.

        :param fade: True if a fade should be performed (when supported)
        :type fade: bool
        """

    def fadein(self, duration, fade_type):
        """Fade-in the cue.

        :param duration: How much the fade should be long (in seconds)
        :type duration: float
        :param fade_type: The fade type
        :type fade_type: FadeInType
        """

    def fadeout(self, duration, fade_type):
        """Fade-out the cue.

        :param duration: How much the fade should be long (in seconds)
        :type duration: float
        :param fade_type: The fade type
        :type fade_type: FadeOutType
        """

    def _ended(self):
        """Remove the Running state, if needed set it to Stop."""
        locked = self._st_lock.acquire(blocking=False)

        if self._state == CueState.Running:
            self._state = CueState.Stop
        else:
            self._state ^= CueState.Running

        self.end.emit(self)

        if locked:
            self._st_lock.release()

    def _error(self, message, details):
        """Remove Running/Pause/Stop state and add Error state."""
        locked = self._st_lock.acquire(blocking=False)

        self._state = (
            (self._state ^ CueState.Running) &
            (self._state ^ CueState.Pause) &
            (self._state ^ CueState.Stop)
        ) | CueState.Error

        self.error.emit(self, message, details)

        if locked:
            self._st_lock.release()

    def current_time(self):
        """Return the current execution time if available, otherwise 0.

        :rtype: int
        """
        return 0

    def prewait_time(self):
        return self._prewait.current_time()

    def postwait_time(self):
        return self._postwait.current_time()

    @property
    def state(self):
        """Return the current state.

        :rtype: int
        """
        return self._state

    def __next_action_changed(self, next_action):
        self.end.disconnect(self.next.emit)
        if next_action == CueNextAction.AutoFollow:
            self.end.connect(self.next.emit)



# -*- coding: utf-8 -*-
#
# This file is part of Linux Show Player
#
# Copyright 2012-2016 Francesco Ceruti <ceppofrancy@gmail.com>
#
# Linux Show Player is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Linux Show Player is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Linux Show Player.  If not, see <http://www.gnu.org/licenses/>.

from PyQt5.QtWidgets import QTimeEdit


class QStepTimeEdit(QTimeEdit):

    def __init__(self, parent=None, **kwargs):
        super().__init__(parent, **kwargs)

        self._sections_steps = {}

    def setSectionStep(self, section, step):
        if not isinstance(section, QTimeEdit.Section):
            raise AttributeError('Not a QTimeEdit.Section')
        if step <= 0:
            raise AttributeError('Step value must be >= 0')

        self._sections_steps[section] = step

    def stepBy(self, value):
        step = self._sections_steps.get(self.currentSection(), 1)

        if abs(value) != step:
            sign = 1 if value >= 0 else -1
            value = sign * step

        super().stepBy(value)



# -*- coding: utf-8 -*-
#
# This file is part of Linux Show Player
#
# Copyright 2012-2016 Francesco Ceruti <ceppofrancy@gmail.com>
#
# Linux Show Player is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Linux Show Player is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Linux Show Player.  If not, see <http://www.gnu.org/licenses/>.

from lisp.core.signal import Signal


class Protocol:
    """Base interface for protocols.

    The init() and reset() functions are called when the respective functions
    of the main-plugin are called.

    When an event that can trigger a cue is "detected", the protocol_event
    signal should be emitted with the event representation.

    To be loaded correctly the class should follow the ClassesLoader
    specification.
    To define the settings, only define a class with same name plus 'Settings'
    as suffix (e.g. Protocol -> ProtocolSettings), in the same file.
    """

    def __init__(self):
        self.protocol_event = Signal()

    def init(self):
        pass

    def reset(self):
        pass



# -*- coding: utf-8 -*-
#
# This file is part of Linux Show Player
#
# Copyright 2012-2016 Francesco Ceruti <ceppofrancy@gmail.com>
#
# Linux Show Player is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Linux Show Player is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Linux Show Player.  If not, see <http://www.gnu.org/licenses/>.

from PyQt5.QtCore import QT_TRANSLATE_NOOP

from lisp.backend.media_element import ElementType, MediaType
from lisp.core.has_properties import Property
from lisp.modules.gst_backend.gi_repository import Gst
from lisp.modules.gst_backend.gst_element import GstMediaElement


class UserElement(GstMediaElement):
    ElementType = ElementType.Plugin
    MediaType = MediaType.Audio
    Name = QT_TRANSLATE_NOOP('MediaElementName', 'Custom Element')

    bin = Property(default='')

    def __init__(self, pipeline):
        super().__init__()

        self.pipeline = pipeline
        self.audio_convert_sink = Gst.ElementFactory.make("audioconvert", None)
        # A default assignment for the bin
        self.gst_bin = Gst.ElementFactory.make("identity", None)
        self.gst_bin.set_property("signal-handoffs", False)
        self.audio_convert_src = Gst.ElementFactory.make("audioconvert", None)

        pipeline.add(self.audio_convert_sink)
        pipeline.add(self.gst_bin)
        pipeline.add(self.audio_convert_src)

        self.audio_convert_sink.link(self.gst_bin)
        self.gst_bin.link(self.audio_convert_src)

        self._old_bin = self.gst_bin
        self.changed('bin').connect(self.__prepare_bin)

    def sink(self):
        return self.audio_convert_sink

    def src(self):
        return self.audio_convert_src

    def __prepare_bin(self, value):
        if value != '' and value != self._old_bin:
            self._old_bin = value

            # If in playing we need to restart the pipeline after unblocking
            playing = self.gst_bin.current_state == Gst.State.PLAYING
            # Block the stream
            pad = self.audio_convert_sink.sinkpads[0]
            probe = pad.add_probe(Gst.PadProbeType.BLOCK, lambda *a: 0, "")

            # Unlink the components
            self.audio_convert_sink.unlink(self.gst_bin)
            self.gst_bin.unlink(self.audio_convert_src)
            self.pipeline.remove(self.gst_bin)

            # Create the bin, when fail use a do-nothing element
            try:
                self.gst_bin = Gst.parse_bin_from_description(value, True)
            except Exception:
                self.gst_bin = Gst.ElementFactory.make("identity", None)
                self.gst_bin.set_property("signal-handoffs", False)

            # Link the components
            self.pipeline.add(self.gst_bin)
            self.audio_convert_sink.link(self.gst_bin)
            self.gst_bin.link(self.audio_convert_src)

            # Unblock the stream
            pad.remove_probe(probe)
            if playing:
                self.pipeline.set_state(Gst.State.PLAYING)


# -*- coding: utf-8 -*-
#
# This file is part of Linux Show Player
#
# Copyright 2012-2016 Francesco Ceruti <ceppofrancy@gmail.com>
#
# Linux Show Player is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Linux Show Player is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Linux Show Player.  If not, see <http://www.gnu.org/licenses/>.

from urllib.request import unquote

from PyQt5 import QtCore
from PyQt5.QtWidgets import QAction, QMessageBox, QDialog, QVBoxLayout, \
    QTreeWidget, QAbstractItemView, QDialogButtonBox, QTreeWidgetItem
from PyQt5.QtWidgets import QHeaderView

from lisp.application import Application
from lisp.modules.gst_backend.gst_utils import gst_uri_metadata, \
    gst_parse_tag_list
from lisp.core.module import Module
from lisp.cues.media_cue import MediaCue
from lisp.layouts.cue_layout import CueLayout
from lisp.ui.mainwindow import MainWindow
from lisp.ui.ui_utils import translate


class MediaInfo(Module):
    Name = 'MediaInfo'

    def __init__(self):
        self.menuAction = QAction(None)
        self.menuAction.triggered.connect(self.show_info)
        self.menuAction.setText(translate('MediaInfo', 'Media Info'))

        CueLayout.cm_registry.add_separator(MediaCue)
        CueLayout.cm_registry.add_item(self.menuAction, MediaCue)

    def show_info(self, clicked):
        media_uri = Application().layout.get_context_cue().media.input_uri()
        if not media_uri:
            QMessageBox.critical(MainWindow(), translate('MediaInfo', 'Error'),
                                 translate('MediaInfo', 'No info to display'))
        else:
            gst_info = gst_uri_metadata(media_uri)
            info = {'URI': unquote(gst_info.get_uri())}

            # Audio streams info
            for stream in gst_info.get_audio_streams():
                name = stream.get_stream_type_nick().capitalize()
                info[name] = {
                    'Bitrate': str(stream.get_bitrate() // 1000) + ' Kb/s',
                    'Channels': str(stream.get_channels()),
                    'Sample rate': str(stream.get_sample_rate()) + ' Hz',
                    'Sample size': str(stream.get_depth()) + ' bit'
                }

            # Video streams info
            for stream in gst_info.get_video_streams():
                name = stream.get_stream_type_nick().capitalize()
                info[name] = {
                    'Height': str(stream.get_height()) + ' px',
                    'Width': str(stream.get_width()) + ' px',
                    'Framerate': str(round(stream.get_framerate_num() /
                                           stream.get_framerate_denom()))
                }

            # Media tags
            info['Tags'] = {}
            tags = gst_parse_tag_list(gst_info.get_tags())
            for tag_name in tags:
                if type(tags[tag_name]).__str__ is not object.__str__:
                    info['Tags'][tag_name.capitalize()] = str(tags[tag_name])

            if not info['Tags']:
                info.pop('Tags')

            # Show the dialog
            dialog = InfoDialog(MainWindow(), info,
                                Application().layout.get_context_cue().name)
            dialog.exec_()


class InfoDialog(QDialog):
    def __init__(self, parent, info, title):
        super().__init__(parent)

        self.setWindowTitle(
            translate('MediaInfo', 'Media Info') + ' - ' + title)
        self.setWindowModality(QtCore.Qt.ApplicationModal)
        self.setMinimumSize(550, 300)
        self.resize(550, 500)

        self.vLayout = QVBoxLayout(self)

        self.infoTree = QTreeWidget(self)
        self.infoTree.setColumnCount(2)
        self.infoTree.setHeaderLabels([translate('MediaInfo', 'Info'),
                                       translate('MediaInfo', 'Value')])
        self.infoTree.setAlternatingRowColors(True)
        self.infoTree.setSelectionMode(QAbstractItemView.NoSelection)
        self.infoTree.setEditTriggers(QAbstractItemView.NoEditTriggers)
        self.infoTree.header().setStretchLastSection(False)
        self.infoTree.header().setSectionResizeMode(
            QHeaderView.ResizeToContents)
        self.vLayout.addWidget(self.infoTree)

        self.__generate_items(info)
        self.infoTree.expandAll()

        self.buttonBox = QDialogButtonBox(self)
        self.buttonBox.setStandardButtons(QDialogButtonBox.Close)
        self.vLayout.addWidget(self.buttonBox)

        self.buttonBox.rejected.connect(self.close)

    def __generate_items(self, info, parent=None):
        for key in sorted(info.keys()):
            if isinstance(info[key], dict):
                widget = QTreeWidgetItem([key])
                self.__generate_items(info[key], widget)
            else:
                widget = QTreeWidgetItem([key, info[key]])

            if parent is not None:
                parent.addChild(widget)
            else:
                self.infoTree.addTopLevelItem(widget)



# -*- coding: utf-8 -*-
#
# This file is part of Linux Show Player
#
# Copyright 2012-2016 Francesco Ceruti <ceppofrancy@gmail.com>
#
# Linux Show Player is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Linux Show Player is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Linux Show Player.  If not, see <http://www.gnu.org/licenses/>.

import logging
import os.path
import re
import traceback

try:
    from os import scandir
except ImportError:
    from scandir import scandir


class load_classes:
    """Generator for iterating over classes in a package.

    The class name must be the same as the module name, optionally
    suffixes and prefixes lists can be provided.

    .. highlight::

        modules
        ├── module1.py
        |   └──Module1
        └── module_extra.py
            └── ModuleExtra

    """

    def __init__(self, pkg, pkg_path, pre=('',), suf=('',), exclude=()):
        """
        :param pkg: dotted name of the package
        :param pkg_path: path of the package to scan
        :param pre: iterable of prefixes
        :param suf: iterable suffixes
        :param exclude: iterable of excluded modules names

        `pre` and `suf` must have the same length to work correctly, if some of
        the classes to load have no prefix/suffix an empty one should be used.
        """
        self.pkg = pkg
        self.prefixes = pre
        self.suffixes = suf
        self.excluded = exclude
        self.pkg_path = pkg_path

    def __iter__(self):
        return self.load()

    def load(self):
        """Generate lists of tuples (class-name, class-object)."""
        for entry in scandir(self.pkg_path):

            # Exclude __init__, __pycache__ and likely
            if re.match('^__.*', entry.name):
                continue

            mod_name = entry.name
            if entry.is_file():
                # Split filename and extension
                mod_name, ext = os.path.splitext(entry.name)

                # Exclude all non-python files
                if not re.match('.py[cod]?', ext):
                    continue

            # Exclude excluded ¯\_(ツ)_/¯
            if mod_name in self.excluded:
                continue

            mod_path = self.pkg + '.' + mod_name

            try:
                # Import module
                module = import_module(mod_path)

                # Load class from imported module
                for prefix, suffix in zip(self.prefixes, self.suffixes):
                    name = self._class_name(mod_name, prefix, suffix)
                    if hasattr(module, name):
                        cls = getattr(module, name)
                        yield (name, cls)

            except ImportError:
                logging.warning('Cannot load module: {0}'.format(mod_name))
                logging.debug(traceback.format_exc())

    @staticmethod
    def _class_name(mod_name, pre='', suf=''):
        """Return the supposed class name from loaded module.

        Substitutions:
         * Remove `underscores`
         * First letters to uppercase version

        For example:
         * For "module", the result will be "Module"
         * For "module_name", the result will be "ModuleName"
        """

        # Capitalize the first letter of each word
        base_name = ''.join(word.title() for word in mod_name.split('_'))
        # Add prefix and suffix to the base name
        return pre + base_name + suf


def import_module(module_path):
    return __import__(module_path, globals(), locals(), ['*'])



# -*- coding: utf-8 -*-
#
# This file is part of Linux Show Player
#
# Copyright 2012-2016 Francesco Ceruti <ceppofrancy@gmail.com>
#
# Linux Show Player is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Linux Show Player is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Linux Show Player.  If not, see <http://www.gnu.org/licenses/>.

from os.path import dirname

from lisp.core.loading import load_classes

# Use a set() for avoiding duplication
__PAGES = set()


def load():
    for _, page in load_classes(__package__,
                                dirname(__file__),
                                suf=('Settings', )):
        __PAGES.add(page)


def pages():
    return list(__PAGES)


def pages_by_element():
    return {s.ELEMENT.__name__: s for s in __PAGES}



# -*- coding: utf-8 -*-
#
# This file is part of Linux Show Player
#
# Copyright 2012-2016 Francesco Ceruti <ceppofrancy@gmail.com>
#
# Linux Show Player is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Linux Show Player is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Linux Show Player.  If not, see <http://www.gnu.org/licenses/>.

from PyQt5.QtCore import Qt, QT_TRANSLATE_NOOP
from PyQt5.QtWidgets import QVBoxLayout, QGroupBox

from lisp.modules.gst_backend.gst_pipe_edit import GstPipeEdit
from lisp.ui.settings.settings_page import SettingsPage
from lisp.ui.ui_utils import translate


class GstSettings(SettingsPage):
    Name = QT_TRANSLATE_NOOP('SettingsPageName', 'GStreamer settings')

    def __init__(self, **kwargs):
        super().__init__()
        self.setLayout(QVBoxLayout())
        self.layout().setAlignment(Qt.AlignTop)

        self.pipeGroup = QGroupBox(self)
        self.pipeGroup.setLayout(QVBoxLayout())
        self.layout().addWidget(self.pipeGroup)

        self.pipeEdit = GstPipeEdit([], app_mode=True)
        self.pipeGroup.layout().addWidget(self.pipeEdit)

        self.retranslateUi()

    def retranslateUi(self):
        self.pipeGroup.setTitle(translate('GstSettings', 'Pipeline'))

    def get_settings(self):
        return {'Gst': {'pipeline': ', '.join(self.pipeEdit.get_pipe())}}

    def load_settings(self, settings):
        pipe = tuple(settings['Gst']['pipeline'].replace(' ', '').split(','))
        self.pipeEdit.set_pipe(pipe)



# -*- coding: utf-8 -*-
#
# This file is part of Linux Show Player
#
# Copyright 2012-2016 Francesco Ceruti <ceppofrancy@gmail.com>
#
# Linux Show Player is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Linux Show Player is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Linux Show Player.  If not, see <http://www.gnu.org/licenses/>.

from PyQt5.QtCore import Qt, QMimeData, pyqtSignal, QPoint
from PyQt5.QtGui import QColor, QDrag
from PyQt5.QtWidgets import QProgressBar, QLCDNumber, QLabel, QHBoxLayout, \
    QWidget, QGridLayout, QSizePolicy

from lisp.backend.audio_utils import slider_to_fader, fader_to_slider
from lisp.core.signal import Connection
from lisp.core.util import strtime
from lisp.cues.cue import CueState
from lisp.cues.cue_time import CueTime
from lisp.cues.media_cue import MediaCue
from lisp.layouts.cart_layout.page_widget import PageWidget
from lisp.ui.ui_utils import pixmap_from_icon
from lisp.ui.widgets import QClickLabel, QClickSlider, QDbMeter,\
    QDetailedMessageBox


class CueWidget(QWidget):
    ICON_SIZE = 14
    SLIDER_RANGE = 1000

    context_menu_request = pyqtSignal(object, QPoint)
    edit_request = pyqtSignal(object)
    cue_executed = pyqtSignal(object)

    def __init__(self, cue, **kwargs):
        super().__init__(**kwargs)
        self.cue = None

        self._selected = False
        self._accurate_timing = False
        self._show_dbmeter = False
        self._show_volume = False
        self._countdown_mode = True

        self._dbmeter_element = None
        self._fade_element = None
        self._volume_element = None

        self.setAttribute(Qt.WA_TranslucentBackground)
        self.setLayout(QGridLayout())

        self.layout().setContentsMargins(0, 0, 0, 0)
        self.layout().setSpacing(2)
        self.layout().setColumnStretch(0, 6)
        self.layout().setRowStretch(0, 4)

        self.nameButton = QClickLabel(self)
        self.nameButton.setObjectName('ButtonCueWidget')
        self.nameButton.setWordWrap(True)
        self.nameButton.setAlignment(Qt.AlignCenter)
        self.nameButton.setFocusPolicy(Qt.NoFocus)
        self.nameButton.clicked.connect(self._clicked)
        self.nameButton.setSizePolicy(QSizePolicy.Ignored, QSizePolicy.Ignored)
        self.layout().addWidget(self.nameButton, 0, 0)

        self.statusIcon = QLabel(self.nameButton)
        self.statusIcon.setStyleSheet('background-color: transparent')
        self.statusIcon.setPixmap(
            pixmap_from_icon('led-off', CueWidget.ICON_SIZE))

        self.seekSlider = QClickSlider(self.nameButton)
        self.seekSlider.setOrientation(Qt.Horizontal)
        self.seekSlider.setFocusPolicy(Qt.NoFocus)
        self.seekSlider.setVisible(False)

        self.volumeSlider = QClickSlider(self.nameButton)
        self.volumeSlider.setObjectName('VolumeSlider')
        self.volumeSlider.setOrientation(Qt.Vertical)
        self.volumeSlider.setFocusPolicy(Qt.NoFocus)
        self.volumeSlider.setRange(0, CueWidget.SLIDER_RANGE)
        self.volumeSlider.setPageStep(10)
        self.volumeSlider.valueChanged.connect(
            self._change_volume, Qt.DirectConnection)
        self.volumeSlider.setVisible(False)

        self.dbMeter = QDbMeter(self)
        self.dbMeter.setSizePolicy(QSizePolicy.Ignored, QSizePolicy.Ignored)
        self.dbMeter.setVisible(False)

        self.timeBar = QProgressBar(self)
        self.timeBar.setTextVisible(False)
        self.timeBar.setLayout(QHBoxLayout())
        self.timeBar.layout().setContentsMargins(0, 0, 0, 0)
        self.timeDisplay = QLCDNumber(self.timeBar)
        self.timeDisplay.setStyleSheet('background-color: transparent')
        self.timeDisplay.setSegmentStyle(QLCDNumber.Flat)
        self.timeDisplay.setDigitCount(8)
        self.timeDisplay.display('00:00:00')
        self.timeBar.layout().addWidget(self.timeDisplay)
        self.timeBar.setSizePolicy(QSizePolicy.Ignored, QSizePolicy.Ignored)
        self.timeBar.setVisible(False)

        self._set_cue(cue)

    @property
    def selected(self):
        return self._selected

    @selected.setter
    def selected(self, value):
        self._selected = value
        self._update_style(self.cue.stylesheet)

    def contextMenuEvent(self, event):
        self.context_menu_request.emit(self, event.globalPos())

    def mouseMoveEvent(self, event):
        if (event.buttons() == Qt.LeftButton and
                (event.modifiers() == Qt.ControlModifier or
                         event.modifiers() == Qt.ShiftModifier)):
            mime_data = QMimeData()
            mime_data.setText(PageWidget.DRAG_MAGIC)

            drag = QDrag(self)
            drag.setMimeData(mime_data)
            drag.setPixmap(self.grab(self.rect()))

            if event.modifiers() == Qt.ControlModifier:
                drag.exec_(Qt.MoveAction)
            else:
                drag.exec_(Qt.CopyAction)

            event.accept()
        else:
            event.ignore()

    def set_countdown_mode(self, mode):
        self._countdown_mode = mode
        self._update_time(self.cue.current_time())

    def set_accurate_timing(self, enable):
        self._accurate_timing = enable
        if self.cue.state & CueState.Pause:
            self._update_time(self.cue.current_time(), True)
        elif not self.cue.state & CueState.Running:
            self._update_duration(self.cue.duration)

    def show_seek_slider(self, visible):
        if isinstance(self.cue, MediaCue):
            self.seekSlider.setVisible(visible)
            self.update()

    def show_dbmeters(self, visible):
        if isinstance(self.cue, MediaCue):
            self._show_dbmeter = visible

            if self._dbmeter_element is not None:
                self._dbmeter_element.level_ready.disconnect(self.dbMeter.plot)
                self._dbmeter_element = None

            if visible:
                self._dbmeter_element = self.cue.media.element('DbMeter')
                if self._dbmeter_element is not None:
                    self._dbmeter_element.level_ready.connect(self.dbMeter.plot)

                self.layout().addWidget(self.dbMeter, 0, 2)
                self.layout().setColumnStretch(2, 1)
                self.dbMeter.show()
            else:
                self.layout().removeWidget(self.dbMeter)
                self.layout().setColumnStretch(2, 0)
                self.dbMeter.hide()

            self.update()

    def show_volume_slider(self, visible):
        if isinstance(self.cue, MediaCue):
            self._show_volume = visible

            if self._volume_element is not None:
                self._volume_element.changed('volume').disconnect(
                    self.reset_volume)
                self._volume_element = None

            if visible:
                self.volumeSlider.setEnabled(self.cue.state & CueState.Running)
                self._volume_element = self.cue.media.element('Volume')
                if self._volume_element is not None:
                    self.reset_volume()
                    self._volume_element.changed('volume').connect(
                        self.reset_volume,
                        Connection.QtQueued)

                self.layout().addWidget(self.volumeSlider, 0, 1)
                self.layout().setColumnStretch(1, 1)
                self.volumeSlider.show()
            else:
                self.layout().removeWidget(self.volumeSlider)
                self.layout().setColumnStretch(1, 0)
                self.volumeSlider.hide()

            self.update()

    def reset_volume(self):
        if self._volume_element is not None:
            self.volumeSlider.setValue(round(fader_to_slider(
                self._volume_element.volume) * CueWidget.SLIDER_RANGE))

    def _set_cue(self, cue):
        self.cue = cue

        # Cue properties changes
        self.cue.changed('name').connect(
            self._update_name, Connection.QtQueued)
        self.cue.changed('stylesheet').connect(
            self._update_style, Connection.QtQueued)
        self.cue.changed('duration').connect(
            self._update_duration, Connection.QtQueued)
        self.cue.changed('description').connect(
            self._update_description, Connection.QtQueued)

        # Fade enter/exit
        self.cue.fadein_start.connect(self._enter_fadein, Connection.QtQueued)
        self.cue.fadein_end.connect(self._exit_fade, Connection.QtQueued)
        self.cue.fadeout_start.connect(self._enter_fadeout, Connection.QtQueued)
        self.cue.fadeout_end.connect(self._exit_fade, Connection.QtQueued)

        # Cue status changed
        self.cue.interrupted.connect(self._status_stopped, Connection.QtQueued)
        self.cue.started.connect(self._status_playing, Connection.QtQueued)
        self.cue.stopped.connect(self._status_stopped, Connection.QtQueued)
        self.cue.paused.connect(self._status_paused, Connection.QtQueued)
        self.cue.error.connect(self._status_error, Connection.QtQueued)
        self.cue.end.connect(self._status_stopped, Connection.QtQueued)

        # DbMeter connections
        if isinstance(cue, MediaCue):
            self.cue.media.elements_changed.connect(
                self._media_updated, Connection.QtQueued)

            self.cue.paused.connect(self.dbMeter.reset, Connection.QtQueued)
            self.cue.stopped.connect(self.dbMeter.reset, Connection.QtQueued)
            self.cue.end.connect(self.dbMeter.reset, Connection.QtQueued)
            self.cue.error.connect(self.dbMeter.reset, Connection.QtQueued)

            self.seekSlider.sliderMoved.connect(self.cue.media.seek)
            self.seekSlider.sliderJumped.connect(self.cue.media.seek)

        self._cue_time = CueTime(self.cue)
        self._cue_time.notify.connect(self._update_time, Connection.QtQueued)

        self._update_name(cue.name)
        self._update_style(cue.stylesheet)
        self._update_duration(self.cue.duration)

    def _media_updated(self):
        self.show_dbmeters(self._show_dbmeter)
        self.show_volume_slider(self._show_volume)

    def _update_name(self, name):
        self.nameButton.setText(name)

    def _update_description(self, description):
        self.nameButton.setToolTip(description)

    def _change_volume(self, new_volume):
        self._volume_element.current_volume = slider_to_fader(
            new_volume / CueWidget.SLIDER_RANGE)

    def _clicked(self, event):
        if not (self.seekSlider.geometry().contains(event.pos()) and
                    self.seekSlider.isVisible()):
            if event.button() != Qt.RightButton:
                if event.modifiers() == Qt.ShiftModifier:
                    self.edit_request.emit(self.cue)
                elif event.modifiers() == Qt.ControlModifier:
                    self.selected = not self.selected
                else:
                    self.cue_executed.emit(self.cue)
                    self.cue.execute()

    def _update_style(self, stylesheet):
        stylesheet += 'text-decoration: underline;' if self.selected else ''
        self.nameButton.setStyleSheet(stylesheet)

    def _enter_fadein(self):
        p = self.timeDisplay.palette()
        p.setColor(p.WindowText, QColor(0, 255, 0))
        self.timeDisplay.setPalette(p)

    def _enter_fadeout(self):
        p = self.timeDisplay.palette()
        p.setColor(p.WindowText, QColor(255, 50, 50))
        self.timeDisplay.setPalette(p)

    def _exit_fade(self):
        self.timeDisplay.setPalette(self.timeBar.palette())

    def _status_stopped(self):
        self.statusIcon.setPixmap(
            pixmap_from_icon('led-off', CueWidget.ICON_SIZE))
        self.volumeSlider.setEnabled(False)
        self._update_time(0, True)
        self.reset_volume()

    def _status_playing(self):
        self.statusIcon.setPixmap(
            pixmap_from_icon('led-running', CueWidget.ICON_SIZE))
        self.volumeSlider.setEnabled(True)

    def _status_paused(self):
        self.statusIcon.setPixmap(
            pixmap_from_icon('led-pause', CueWidget.ICON_SIZE))
        self.volumeSlider.setEnabled(False)

    def _status_error(self, cue, error, details):
        self.statusIcon.setPixmap(
            pixmap_from_icon('led-error', CueWidget.ICON_SIZE))
        self.volumeSlider.setEnabled(False)
        self.reset_volume()

        QDetailedMessageBox.dcritical(self.cue.name, error, details)

    def _update_duration(self, duration):
        # Update the maximum values of seek-slider and time progress-bar
        if duration > 0:
            if not self.timeBar.isVisible():
                self.layout().addWidget(self.timeBar, 1, 0, 1, 3)
                self.layout().setRowStretch(1, 1)
                self.timeBar.show()
            self.timeBar.setMaximum(duration)
            self.seekSlider.setMaximum(duration)
        else:
            self.timeBar.hide()
            self.layout().setRowStretch(1, 0)

        if not self.cue.state & CueState.Running:
            self._update_time(duration, True)

    def _update_time(self, time, ignore_visibility=False):
        if ignore_visibility or not self.visibleRegion().isEmpty():
            # If the given value is the duration or < 0 set the time to 0
            if time == self.cue.duration or time < 0:
                time = 0

            # Set the value the seek slider
            self.seekSlider.setValue(time)

            # If in count-down mode the widget will show the remaining time
            if self._countdown_mode:
                time = self.cue.duration - time

            # Set the value of the timer progress-bar
            if self.cue.duration > 0:
                self.timeBar.setValue(time)

            # Show the time in the widget
            self.timeDisplay.display(
                strtime(time, accurate=self._accurate_timing))

    def resizeEvent(self, event):
        self.update()

    def update(self):
        super().update()
        self.layout().activate()

        s_width = self.nameButton.width() - 8
        s_height = self.seekSlider.height()
        s_ypos = self.nameButton.height() - s_height

        self.seekSlider.setGeometry(4, s_ypos, s_width, s_height)
        self.statusIcon.setGeometry(4, 4,
                                    CueWidget.ICON_SIZE,
                                    CueWidget.ICON_SIZE)



# -*- coding: utf-8 -*-
#
# This file is part of Linux Show Player
#
# Copyright 2012-2016 Francesco Ceruti <ceppofrancy@gmail.com>
#
# Linux Show Player is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Linux Show Player is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Linux Show Player.  If not, see <http://www.gnu.org/licenses/>.

from PyQt5.QtCore import Qt, QT_TRANSLATE_NOOP
from PyQt5.QtWidgets import QGroupBox, QVBoxLayout, QGridLayout, QLabel, \
    QComboBox, QSpinBox, QFrame

from lisp.core.has_properties import Property
from lisp.cues.cue import Cue
from lisp.modules.midi.midi_output import MIDIOutput
from lisp.modules.midi.midi_utils import str_msg_to_dict, dict_msg_to_str
from lisp.ui.settings.cue_settings import CueSettingsRegistry
from lisp.ui.settings.settings_page import SettingsPage
from lisp.ui.ui_utils import translate


class MidiCue(Cue):
    Name = QT_TRANSLATE_NOOP('CueName', 'MIDI Cue')

    message = Property(default='')

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.name = translate('CueName', self.Name)

        midi_out = MIDIOutput()
        if not midi_out.is_open():
            midi_out.open()

    def __start__(self, fade=False):
        if self.message:
            MIDIOutput().send_from_str(self.message)

        return False


class MidiCueSettings(SettingsPage):
    Name = QT_TRANSLATE_NOOP('SettingsPageName', 'MIDI Settings')

    MSGS_ATTRIBUTES = {
        'note_on': ['channel', 'note', 'velocity'],
        'note_off': ['channel', 'note', 'velocity'],
        'control_change': ['channel', 'control', 'value'],
        'program_change': ['channel', 'program', None],
        'polytouch': ['channel', 'note', 'value'],
        'pitchwheel': ['channel', 'pitch', None],
        'song_select': ['song', None, None],
        'songpos': ['pos', None, None],
        'start': [None] * 3,
        'stop': [None] * 3,
        'continue': [None] * 3,
    }

    ATTRIBUTES_RANGE = {
        'channel': (1, 16, -1), 'note': (0, 127, 0),
        'velocity': (0, 127, 0), 'control': (0, 127, 0),
        'program': (0, 127, 0), 'value': (0, 127, 0),
        'song': (0, 127, 0), 'pitch': (-8192, 8191, 0),
        'pos': (0, 16383, 0)
    }

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.setLayout(QVBoxLayout())
        self.layout().setAlignment(Qt.AlignTop)

        self.msgGroup = QGroupBox(self)
        self.msgGroup.setLayout(QGridLayout())
        self.layout().addWidget(self.msgGroup)

        # Message type
        self.msgTypeLabel = QLabel(self.msgGroup)
        self.msgGroup.layout().addWidget(self.msgTypeLabel, 0, 0)
        self.msgTypeCombo = QComboBox(self.msgGroup)
        self.msgTypeCombo.addItems(sorted(self.MSGS_ATTRIBUTES.keys()))
        self.msgTypeCombo.currentTextChanged.connect(self.__type_changed)
        self.msgGroup.layout().addWidget(self.msgTypeCombo, 0, 1)

        line = QFrame(self.msgGroup)
        line.setFrameShape(QFrame.HLine)
        line.setFrameShadow(QFrame.Sunken)
        self.msgGroup.layout().addWidget(line, 1, 0, 1, 2)

        # Data widgets
        self._data_widgets = []
        for n in range(2, 5):
            dataLabel = QLabel(self.msgGroup)
            dataSpin = QSpinBox(self.msgGroup)

            self.msgGroup.layout().addWidget(dataLabel, n, 0)
            self.msgGroup.layout().addWidget(dataSpin, n, 1)

            self._data_widgets.append((dataLabel, dataSpin))

        self.__type_changed(self.msgTypeCombo.currentText())

        self.retranslateUi()

    def retranslateUi(self):
        self.msgGroup.setTitle(translate('MIDICue', 'MIDI Message'))
        self.msgTypeLabel.setText(translate('MIDICue', 'Message type'))

    def __type_changed(self, msg_type):
        for label, spin, attr_name in self.__attributes(msg_type):
            if attr_name is None:
                label.setEnabled(False)
                label.setText('')
                spin.setEnabled(False)
            else:
                label.setEnabled(True)
                label.setText(attr_name.title())

                spin.setEnabled(True)
                spin.setRange(
                    *self.ATTRIBUTES_RANGE.get(attr_name, (0, 0, 0))[0:2])

    def get_settings(self):
        msg_type = self.msgTypeCombo.currentText()
        msg_dict = {'type': msg_type}

        for label, spin, attr_name in self.__attributes(msg_type):
            if spin.isEnabled():
                offset = self.ATTRIBUTES_RANGE.get(attr_name, (0, 0, 0))[2]
                msg_dict[attr_name] = spin.value() + offset

        return {'message': dict_msg_to_str(msg_dict)}

    def __attributes(self, msg_type):
        for (label, spin), attr in zip(self._data_widgets,
                                       self.MSGS_ATTRIBUTES[msg_type]):
            yield label, spin, attr

    def load_settings(self, settings):
        str_msg = settings.get('message', '')
        if str_msg:
            dict_msg = str_msg_to_dict(str_msg)
            self.msgTypeCombo.setCurrentText(dict_msg['type'])

            for label, spin, attr_name in self.__attributes(dict_msg['type']):
                offset = self.ATTRIBUTES_RANGE.get(attr_name, (0, 0, 0))[2]
                spin.setValue(dict_msg.get(label.text().lower(), 0) - offset)


CueSettingsRegistry().add_item(MidiCueSettings, MidiCue)



# -*- coding: utf-8 -*-
#
# This file is part of Linux Show Player
#
# Copyright 2012-2017 Francesco Ceruti <ceppofrancy@gmail.com>
#
# Linux Show Player is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Linux Show Player is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Linux Show Player.  If not, see <http://www.gnu.org/licenses/>.

from PyQt5.QtCore import Qt
from PyQt5.QtGui import QKeySequence
from PyQt5.QtWidgets import QDoubleSpinBox
from PyQt5.QtWidgets import QGroupBox, QVBoxLayout, QCheckBox, QComboBox, \
    QHBoxLayout, QLabel, QKeySequenceEdit, QGridLayout

from lisp.ui.settings.settings_page import SettingsPage
from lisp.ui.ui_utils import translate
from lisp.ui.widgets import FadeComboBox


class ListLayoutSettings(SettingsPage):
    Name = 'List Layout'

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.setLayout(QVBoxLayout())
        self.layout().setAlignment(Qt.AlignTop)

        self.behaviorsGroup = QGroupBox(self)
        self.behaviorsGroup.setLayout(QVBoxLayout())
        self.layout().addWidget(self.behaviorsGroup)

        self.showPlaying = QCheckBox(self.behaviorsGroup)
        self.behaviorsGroup.layout().addWidget(self.showPlaying)

        self.showDbMeters = QCheckBox(self.behaviorsGroup)
        self.behaviorsGroup.layout().addWidget(self.showDbMeters)

        self.showAccurate = QCheckBox(self.behaviorsGroup)
        self.behaviorsGroup.layout().addWidget(self.showAccurate)

        self.showSeek = QCheckBox(self.behaviorsGroup)
        self.behaviorsGroup.layout().addWidget(self.showSeek)

        self.autoNext = QCheckBox(self.behaviorsGroup)
        self.behaviorsGroup.layout().addWidget(self.autoNext)

        self.endListLayout = QHBoxLayout()
        self.behaviorsGroup.layout().addLayout(self.endListLayout)
        self.endListLabel = QLabel(self.behaviorsGroup)
        self.endListLayout.addWidget(self.endListLabel)
        self.endListBehavior = QComboBox(self.behaviorsGroup)
        self.endListBehavior.addItem(translate('ListLayout', 'Stop'), 'Stop')
        self.endListBehavior.addItem(translate('ListLayout', 'Restart'),
                                     'Restart')
        self.endListLayout.addWidget(self.endListBehavior)
        self.endListLayout.setStretch(0, 2)
        self.endListLayout.setStretch(1, 5)

        self.goKeyLayout = QHBoxLayout()
        self.behaviorsGroup.layout().addLayout(self.goKeyLayout)
        self.goKeyLabel = QLabel(self.behaviorsGroup)
        self.goKeyLayout.addWidget(self.goKeyLabel)
        self.goKeyEdit = QKeySequenceEdit(self.behaviorsGroup)
        self.goKeyLayout.addWidget(self.goKeyEdit)
        self.goKeyLayout.setStretch(0, 2)
        self.goKeyLayout.setStretch(1, 5)

        self.useFadeGroup = QGroupBox(self)
        self.useFadeGroup.setLayout(QGridLayout())
        self.layout().addWidget(self.useFadeGroup)

        # per-cue
        self.stopCueFade = QCheckBox(self.useFadeGroup)
        self.useFadeGroup.layout().addWidget(self.stopCueFade, 0, 0)
        self.pauseCueFade = QCheckBox(self.useFadeGroup)
        self.useFadeGroup.layout().addWidget(self.pauseCueFade, 1, 0)
        self.restartCueFade = QCheckBox(self.useFadeGroup)
        self.useFadeGroup.layout().addWidget(self.restartCueFade, 2, 0)
        self.interruptCueFade = QCheckBox(self.useFadeGroup)
        self.useFadeGroup.layout().addWidget(self.interruptCueFade, 3, 0)

        # all
        self.stopAllFade = QCheckBox(self.useFadeGroup)
        self.useFadeGroup.layout().addWidget(self.stopAllFade, 0, 1)
        self.pauseAllFade = QCheckBox(self.useFadeGroup)
        self.useFadeGroup.layout().addWidget(self.pauseAllFade, 1, 1)
        self.restartAllFade = QCheckBox(self.useFadeGroup)
        self.useFadeGroup.layout().addWidget(self.restartAllFade, 2, 1)
        self.interruptAllFade = QCheckBox(self.useFadeGroup)
        self.useFadeGroup.layout().addWidget(self.interruptAllFade, 3, 1)

        self.retranslateUi()

    def retranslateUi(self):
        self.behaviorsGroup.setTitle(
            translate('ListLayout', 'Default behaviors'))
        self.showPlaying.setText(translate('ListLayout', 'Show playing cues'))
        self.showDbMeters.setText(translate('ListLayout', 'Show dB-meters'))
        self.showAccurate.setText(translate('ListLayout', 'Show accurate time'))
        self.showSeek.setText(translate('ListLayout', 'Show seek-bars'))
        self.autoNext.setText(translate('ListLayout', 'Auto-select next cue'))
        self.endListLabel.setText(translate('ListLayout', 'At list end:'))
        self.goKeyLabel.setText(translate('ListLayout', 'Go key:'))

        self.useFadeGroup.setTitle(translate('ListLayout', 'Use fade'))
        self.stopCueFade.setText(translate('ListLayout', 'Stop Cue'))
        self.pauseCueFade.setText(translate('ListLayout', 'Pause Cue'))
        self.restartCueFade.setText(translate('ListLayout', 'Restart Cue'))
        self.interruptCueFade.setText(translate('ListLayout', 'Interrupt Cue'))
        self.stopAllFade.setText(translate('ListLayout', 'Stop All'))
        self.pauseAllFade.setText(translate('ListLayout', 'Pause All'))
        self.restartAllFade.setText(translate('ListLayout', 'Restart All'))
        self.interruptAllFade.setText(translate('ListLayout', 'Interrupt All'))

    def get_settings(self):
        settings = {
            'showplaying': str(self.showPlaying.isChecked()),
            'showdbmeters': str(self.showDbMeters.isChecked()),
            'showseek': str(self.showSeek.isChecked()),
            'showaccurate': str(self.showAccurate.isChecked()),
            'autocontinue': str(self.autoNext.isChecked()),
            'endlist': str(self.endListBehavior.currentData()),
            'gokey': self.goKeyEdit.keySequence().toString(
                QKeySequence.NativeText),
            'stopcuefade': str(self.stopCueFade.isChecked()),
            'pausecuefade': str(self.pauseCueFade.isChecked()),
            'restartcuefade': str(self.restartCueFade.isChecked()),
            'interruptcuefade': str(self.interruptCueFade.isChecked()),
            'stopallfade': str(self.stopAllFade.isChecked()),
            'pauseallfade': str(self.pauseAllFade.isChecked()),
            'restartallfade': str(self.restartAllFade.isChecked()),
            'interruptallfade': str(self.interruptAllFade.isChecked()),
        }

        return {'ListLayout': settings}

    def load_settings(self, settings):
        settings = settings.get('ListLayout', {})

        self.showPlaying.setChecked(settings.get('showplaying') == 'True')
        self.showDbMeters.setChecked(settings.get('showdbmeters') == 'True')
        self.showAccurate.setChecked(settings.get('showaccurate') == 'True')
        self.showSeek.setChecked(settings.get('showseek') == 'True')
        self.autoNext.setChecked(settings.get('autocontinue') == 'True')
        self.endListBehavior.setCurrentText(
            translate('ListLayout', settings.get('endlist', '')))
        self.goKeyEdit.setKeySequence(
            QKeySequence(settings.get('gokey', 'Space'),
                         QKeySequence.NativeText))

        self.stopCueFade.setChecked(settings.get('stopcuefade') == 'True')
        self.pauseCueFade.setChecked(settings.get('pausecuefade') == 'True')
        self.restartCueFade.setChecked(settings.get('restartcuefade') == 'True')
        self.interruptCueFade.setChecked(
            settings.get('interruptcuefade') == 'True')

        self.stopAllFade.setChecked(settings.get('stopallfade') == 'True')
        self.pauseAllFade.setChecked(settings.get('pauseallfade') == 'True')
        self.restartAllFade.setChecked(settings.get('restartallfade') == 'True')
        self.interruptAllFade.setChecked(
            settings.get('interruptallfade') == 'True')


# -*- coding: utf-8 -*-
#
# This file is part of Linux Show Player
#
# Copyright 2012-2016 Francesco Ceruti <ceppofrancy@gmail.com>
#
# Linux Show Player is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Linux Show Player is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Linux Show Player.  If not, see <http://www.gnu.org/licenses/>.

from concurrent.futures import ThreadPoolExecutor
from os import path

from PyQt5.QtCore import QT_TRANSLATE_NOOP

from lisp.backend.media_element import MediaType
from lisp.core.decorators import async_in_pool
from lisp.core.has_properties import Property
from lisp.modules.gst_backend.gi_repository import Gst
from lisp.modules.gst_backend.gst_element import GstProperty, \
    GstSrcElement
from lisp.modules.gst_backend.gst_utils import gst_uri_duration


class UriInput(GstSrcElement):
    MediaType = MediaType.Audio
    Name = QT_TRANSLATE_NOOP('MediaElementName', 'URI Input')

    uri = GstProperty('decoder', default='')
    download = GstProperty('decoder', default=False)
    buffer_size = GstProperty('decoder', default=-1, gst_name='buffer-size')
    use_buffering = GstProperty('decoder', default=False,
                                gst_name='use-buffering')
    _mtime = Property(default=-1)

    def __init__(self, pipe):
        super().__init__()

        self.decoder = Gst.ElementFactory.make("uridecodebin", None)
        self.audio_convert = Gst.ElementFactory.make("audioconvert", None)
        self._handler = self.decoder.connect("pad-added", self.__on_pad_added)

        pipe.add(self.decoder)
        pipe.add(self.audio_convert)

        self.changed('uri').connect(self.__uri_changed)

    def input_uri(self):
        return self.uri

    def dispose(self):
        self.decoder.disconnect(self._handler)

    def src(self):
        return self.audio_convert

    def __on_pad_added(self, *args):
        self.decoder.link(self.audio_convert)

    def __uri_changed(self, value):
        # Save the current mtime (file flag for last-change time)
        mtime = self._mtime
        # If the uri is a file, then update the current mtime
        if value.split('://')[0] == 'file':
            if path.exists(value.split('//')[1]):
                self._mtime = path.getmtime(value.split('//')[1])
        else:
            mtime = None

        # If something is changed or the duration is invalid
        if mtime != self._mtime or self.duration < 0:
            self.__duration()

    @async_in_pool(pool=ThreadPoolExecutor(1))
    def __duration(self):
        self.duration = gst_uri_duration(self.uri)
